"""
277.5010.X212
Created 2023-05-19 10:17:35.857645
"""
from .base import *
from .common import *


class ISA_LOOP_ISA(Segment):
    """Interchange Control Header"""
    _segment_name = 'ISA'
    
    isa01: Annotated[I01, Title('Authorization Information Qualifier'), Usage('R'), Position(1), Enumerated(*['00', '03'])]
    
    isa02: Annotated[I02, Title('Authorization Information'), Usage('R'), Position(2)]
    
    isa03: Annotated[I03, Title('Security Information Qualifier'), Usage('R'), Position(3), Enumerated(*['00', '01'])]
    
    isa04: Annotated[I04, Title('Security Information'), Usage('R'), Position(4)]
    
    isa05: Annotated[I05, Title('Interchange ID Qualifier'), Usage('R'), Position(5), Enumerated(*['01', '14', '20', '27', '28', '29', '30', '33', 'ZZ'])]
    
    isa06: Annotated[I06, Title('Interchange Sender ID'), Usage('R'), Position(6)]
    
    isa07: Annotated[I05, Title('Interchange ID Qualifier'), Usage('R'), Position(7), Enumerated(*['01', '14', '20', '27', '28', '29', '30', '33', 'ZZ'])]
    
    isa08: Annotated[I07, Title('Interchange Receiver ID'), Usage('R'), Position(8)]
    
    isa09: Annotated[I08, Title('Interchange Date'), Usage('R'), Position(9)]
    
    isa10: Annotated[I09, Title('Interchange Time'), Usage('R'), Position(10)]
    
    isa11: Annotated[I65, Title('Interchange Control Standards Identifier'), Usage('R'), Position(11)]
    
    isa12: Annotated[I11, Title('Interchange Control Version Number'), Usage('R'), Position(12), Enumerated(*['00501'])]
    
    isa13: Annotated[I12, Title('Interchange Control Number'), Usage('R'), Position(13)]
    
    isa14: Annotated[I13, Title('Acknowledgment Requested'), Usage('R'), Position(14), Enumerated(*['0', '1'])]
    
    isa15: Annotated[I14, Title('Usage Indicator'), Usage('R'), Position(15), Enumerated(*['P', 'T'])]
    
    isa16: Annotated[I15, Title('Component Element Separator'), Usage('R'), Position(16)]


class GS_LOOP_GS(Segment):
    """Functional Group Header"""
    _segment_name = 'GS'
    
    gs01: Annotated[D_479, Title('Functional Identifier Code'), Usage('R'), Position(1), Enumerated(*['RA'])]
    
    gs02: Annotated[D_142, Title("Application Sender's Code"), Usage('R'), Position(2)]
    
    gs03: Annotated[D_124, Title("Application Receiver's Code"), Usage('R'), Position(3)]
    
    gs04: Annotated[D_373, Title('Date'), Usage('R'), Position(4)]
    
    gs05: Annotated[D_337, Title('Time'), Usage('R'), Position(5)]
    
    gs06: Annotated[D_28, Title('Group Control Number'), Usage('R'), Position(6)]
    
    gs07: Annotated[D_455, Title('Responsible Agency Code'), Usage('R'), Position(7), Enumerated(*['X'])]
    
    gs08: Annotated[D_480, Title('Version / Release / Industry Identifier Code'), Usage('R'), Position(8), Enumerated(*['005010X218'])]


class ST_LOOP_ST(Segment):
    """Transaction Set Header"""
    _segment_name = 'ST'
    
    st01: Annotated[D_143, Title('Transaction Set Identifier Code'), Usage('R'), Position(1), Enumerated(*['277'])]
    
    st02: Annotated[D_329, Title('Transaction Set Control Number'), Usage('R'), Position(2)]
    
    st03: Annotated[D_1705, Title('Version, Release, or Industry Identifier'), Usage('R'), Position(3), Enumerated(*['005010X212'])]


class HEADER_BHT(Segment):
    """Beginning of Hierarchical Transaction"""
    _segment_name = 'BHT'
    
    bht01: Annotated[D_1005, Title('Hierarchical Structure Code'), Usage('R'), Position(1), Enumerated(*['0010'])]
    
    bht02: Annotated[D_353, Title('Transaction Set Purpose Code'), Usage('R'), Position(2), Enumerated(*['08'])]
    
    bht03: Annotated[D_127, Title('Originator Application Transaction Identifier'), Usage('R'), Position(3)]
    
    bht04: Annotated[D_373, Title('Transaction Set Creation Date'), Usage('R'), Position(4)]
    
    bht05: Annotated[D_337, Title('Transaction Set Creation Time'), Usage('R'), Position(5)]
    
    bht06: Annotated[D_640, Title('Transaction Type Code'), Usage('R'), Position(6), Enumerated(*['DG'])]


class HEADER(Loop):
    
    bht: Annotated[HEADER_BHT, Title('Beginning of Hierarchical Transaction'), Usage('R'), Position(200), Required(True)]


class L2000A_HL(Segment):
    """Information Source Level"""
    _segment_name = 'HL'
    
    hl01: Annotated[D_628, Title('Hierarchical ID Number'), Usage('R'), Position(1)]
    
    hl02: Annotated[D_734, Title('Hierarchical Parent ID Number'), Usage('N'), Position(2)]
    
    hl03: Annotated[D_735, Title('Hierarchical Level Code'), Usage('R'), Position(3), Enumerated(*['20'])]
    
    hl04: Annotated[D_736, Title('Hierarchical Child Code'), Usage('R'), Position(4), Enumerated(*['1'])]


class L2100A_NM1(Segment):
    """Payer Name"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title('Entity Identifier Code'), Usage('R'), Position(1), Enumerated(*['PR'])]
    
    nm102: Annotated[D_1065, Title('Entity Type Qualifier'), Usage('R'), Position(2), Enumerated(*['2'])]
    
    nm103: Annotated[D_1035, Title('Payer Name'), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title('Name First'), Usage('N'), Position(4)]
    
    nm105: Annotated[D_1037, Title('Name Middle'), Usage('N'), Position(5)]
    
    nm106: Annotated[D_1038, Title('Name Prefix'), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title('Name Suffix'), Usage('N'), Position(7)]
    
    nm108: Annotated[D_66, Title('Identification Code Qualifier'), Usage('R'), Position(8), Enumerated(*['PI', 'XV'])]
    
    nm109: Annotated[D_67, Title('Payer Identifier'), Usage('R'), Position(9)]
    
    nm110: Annotated[D_706, Title('Entity Relationship Code'), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title('Entity Identifier Code'), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title('Name Last or Organization Name'), Usage('N'), Position(12)]


class L2100A_PER(Segment):
    """Payer Contact Information"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title('Contact Function Code'), Usage('R'), Position(1), Enumerated(*['IC'])]
    
    per02: Annotated[D_93, Title('Payer Contact Name'), Usage('S'), Position(2)]
    
    per03: Annotated[D_365, Title('Communication Number Qualifier'), Usage('R'), Position(3), Enumerated(*['ED', 'EM', 'FX', 'TE'])]
    
    per04: Annotated[D_364, Title('Payer Contact Communication Number'), Usage('R'), Position(4)]
    
    per05: Annotated[D_365, Title('Communication Number Qualifier'), Usage('S'), Position(5), Enumerated(*['ED', 'EM', 'EX', 'FX', 'TE'])]
    
    per06: Annotated[D_364, Title('Payer Contact Communication Number'), Usage('S'), Position(6)]
    
    per07: Annotated[D_365, Title('Communication Number Qualifier'), Usage('S'), Position(7), Enumerated(*['ED', 'EM', 'EX', 'FX', 'TE'])]
    
    per08: Annotated[D_364, Title('Payer Contact Communication Number'), Usage('S'), Position(8)]
    
    per09: Annotated[D_443, Title('Contact Inquiry Reference'), Usage('N'), Position(9)]


class L2100A(Loop):
    
    nm1: Annotated[L2100A_NM1, Title('Payer Name'), Usage('R'), Position(500), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    per: Annotated[L2100A_PER, Title('Payer Contact Information'), Usage('S'), Position(800), Syntax(['P0304', 'P0506', 'P0708']), Required(True)]


class L2000B_HL(Segment):
    """Information Receiver Level"""
    _segment_name = 'HL'
    
    hl01: Annotated[D_628, Title('Hierarchical ID Number'), Usage('R'), Position(1)]
    
    hl02: Annotated[D_734, Title('Hierarchical Parent ID Number'), Usage('R'), Position(2)]
    
    hl03: Annotated[D_735, Title('Hierarchical Level Code'), Usage('R'), Position(3), Enumerated(*['21'])]
    
    hl04: Annotated[D_736, Title('Hierarchical Child Code'), Usage('R'), Position(4), Enumerated(*['0', '1'])]


class L2100B_NM1(Segment):
    """Information Receiver Name"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title('Entity Identifier Code'), Usage('R'), Position(1), Enumerated(*['41'])]
    
    nm102: Annotated[D_1065, Title('Entity Type Qualifier'), Usage('R'), Position(2), Enumerated(*['1', '2'])]
    
    nm103: Annotated[D_1035, Title('Information Receiver Last or Organization Name'), Usage('S'), Position(3)]
    
    nm104: Annotated[D_1036, Title('Information Receiver First Name'), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title('Information Receiver Middle Name'), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title('Information Receiver Name Prefix'), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title('Information Receiver Name Suffix'), Usage('N'), Position(7)]
    
    nm108: Annotated[D_66, Title('Identification Code Qualifier'), Usage('R'), Position(8), Enumerated(*['46'])]
    
    nm109: Annotated[D_67, Title('Information Receiver Identification Number'), Usage('R'), Position(9)]
    
    nm110: Annotated[D_706, Title('Entity Relationship Code'), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title('Entity Identifier Code'), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title('Name Last or Organization Name'), Usage('N'), Position(12)]


class L2100B(Loop):
    
    nm1: Annotated[L2100B_NM1, Title('Information Receiver Name'), Usage('R'), Position(500), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]


class L2200B_TRN(Segment):
    """Information Receiver Trace Identifier"""
    _segment_name = 'TRN'
    
    trn01: Annotated[D_481, Title('Trace Type Code'), Usage('R'), Position(1), Enumerated(*['2'])]
    
    trn02: Annotated[D_127, Title('Claim Transaction Batch Number'), Usage('R'), Position(2)]
    
    trn03: Annotated[D_509, Title('Originating Company Identifier'), Usage('N'), Position(3)]
    
    trn04: Annotated[D_127, Title('Reference Identification'), Usage('N'), Position(4)]


class L2200B_STC01(Composite):
    """Health Care Claim Status"""
    
    stc01_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc01_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc01_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*['41', 'AY', 'PR'])]
    
    stc01_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('N'), Position(4)]


class L2200B_STC10(Composite):
    """Health Care Claim Status"""
    
    stc10_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc10_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc10_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3)]
    
    stc10_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('N'), Position(4)]


class L2200B_STC11(Composite):
    """Health Care Claim Status"""
    
    stc11_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc11_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc11_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3)]
    
    stc11_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('N'), Position(4)]


class L2200B_STC(Segment):
    """Information Receiver Status Information"""
    _segment_name = 'STC'
    
    stc01: Annotated[L2200B_STC01, Title('Health Care Claim Status'), Usage('R'), Position(1), Required(True)]
    
    stc02: Annotated[D_373, Title('Status Information Effective Date'), Usage('R'), Position(2)]
    
    stc03: Annotated[D_306, Title('Action Code'), Usage('N'), Position(3)]
    
    stc04: Annotated[D_782, Title('Monetary Amount'), Usage('N'), Position(4)]
    
    stc05: Annotated[D_782, Title('Monetary Amount'), Usage('N'), Position(5)]
    
    stc06: Annotated[D_373, Title('Date'), Usage('N'), Position(6)]
    
    stc07: Annotated[D_591, Title('Payment Method Code'), Usage('N'), Position(7)]
    
    stc08: Annotated[D_373, Title('Date'), Usage('N'), Position(8)]
    
    stc09: Annotated[D_429, Title('Check Number'), Usage('N'), Position(9)]
    
    stc10: Annotated[L2200B_STC10, Title('Health Care Claim Status'), Usage('S'), Position(10), Required(True)]
    
    stc11: Annotated[L2200B_STC11, Title('Health Care Claim Status'), Usage('S'), Position(11), Required(True)]
    
    stc12: Annotated[D_933, Title('Free-form Message Text'), Usage('N'), Position(12)]


class L2200B(Loop):
    
    trn: Annotated[L2200B_TRN, Title('Information Receiver Trace Identifier'), Usage('R'), Position(900), Required(True)]
    ItemStc: TypeAlias = Annotated[L2200B_STC, Title('Information Receiver Status Information'), Usage('R'), Position(1000), Required(True)]
    stc: Annotated[list[ItemStc], MinItems(1)]


class L2000B(Loop):
    
    hl: Annotated[L2000B_HL, Title('Information Receiver Level'), Usage('R'), Position(100), Required(True)]
    ItemL2100B: TypeAlias = Annotated[L2100B, Title('Information Receiver Name'), Usage('R'), Position(500), Required(True)]
    l2100b: Annotated[list[ItemL2100B], MinItems(1)]
    ItemL2200B: TypeAlias = Annotated[L2200B, Title('Information Receiver Trace Identifier'), Usage('S'), Position(900), Required(True)]
    l2200b: Annotated[list[ItemL2200B], MinItems(1)]


class L2000A(Loop):
    
    hl: Annotated[L2000A_HL, Title('Information Source Level'), Usage('R'), Position(100), Required(True)]
    ItemL2100A: TypeAlias = Annotated[L2100A, Title('Payer Name'), Usage('R'), Position(500), Required(True)]
    l2100a: Annotated[list[ItemL2100A], MinItems(1)]
    ItemL2000B: TypeAlias = Annotated[L2000B, Title('Information Receiver Level'), Usage('R'), Position(100), Required(True)]
    l2000b: Annotated[list[ItemL2000B], MinItems(1)]


class L2000C_HL(Segment):
    """Service Provider Level"""
    _segment_name = 'HL'
    
    hl01: Annotated[D_628, Title('Hierarchical ID Number'), Usage('R'), Position(1)]
    
    hl02: Annotated[D_734, Title('Hierarchical Parent ID Number'), Usage('R'), Position(2)]
    
    hl03: Annotated[D_735, Title('Hierarchical Level Code'), Usage('R'), Position(3), Enumerated(*['19'])]
    
    hl04: Annotated[D_736, Title('Hierarchical Child Code'), Usage('R'), Position(4), Enumerated(*['0', '1'])]


class L2100C_NM1(Segment):
    """Provider Name"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title('Entity Identifier Code'), Usage('R'), Position(1), Enumerated(*['1P'])]
    
    nm102: Annotated[D_1065, Title('Entity Type Qualifier'), Usage('R'), Position(2), Enumerated(*['1', '2'])]
    
    nm103: Annotated[D_1035, Title('Provider Last or Organization Name'), Usage('S'), Position(3)]
    
    nm104: Annotated[D_1036, Title('Provider First Name'), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title('Provider Middle Name'), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title('Name Prefix'), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title('Provider Name Suffix'), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title('Identification Code Qualifier'), Usage('R'), Position(8), Enumerated(*['FI', 'SV', 'XX'])]
    
    nm109: Annotated[D_67, Title('Provider Identifier'), Usage('R'), Position(9)]
    
    nm110: Annotated[D_706, Title('Entity Relationship Code'), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title('Entity Identifier Code'), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title('Name Last or Organization Name'), Usage('N'), Position(12)]


class L2100C(Loop):
    
    nm1: Annotated[L2100C_NM1, Title('Provider Name'), Usage('R'), Position(500), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]


class L2200C_TRN(Segment):
    """Provider of Service Trace Identifier"""
    _segment_name = 'TRN'
    
    trn01: Annotated[D_481, Title('Trace Type Code'), Usage('R'), Position(1), Enumerated(*['1'])]
    
    trn02: Annotated[D_127, Title('Provider of Service Information Trace Identifier'), Usage('R'), Position(2)]
    
    trn03: Annotated[D_509, Title('Originating Company Identifier'), Usage('N'), Position(3)]
    
    trn04: Annotated[D_127, Title('Reference Identification'), Usage('N'), Position(4)]


class L2200C_STC01(Composite):
    """Health Care Claim Status"""
    
    stc01_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc01_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc01_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*['1P'])]
    
    stc01_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('N'), Position(4)]


class L2200C_STC10(Composite):
    """Health Care Claim Status"""
    
    stc10_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc10_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc10_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3)]
    
    stc10_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('N'), Position(4)]


class L2200C_STC11(Composite):
    """Health Care Claim Status"""
    
    stc11_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc11_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc11_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3)]
    
    stc11_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('N'), Position(4)]


class L2200C_STC(Segment):
    """Provider Status Information"""
    _segment_name = 'STC'
    
    stc01: Annotated[L2200C_STC01, Title('Health Care Claim Status'), Usage('R'), Position(1), Required(True)]
    
    stc02: Annotated[D_373, Title('Status Information Effective Date'), Usage('R'), Position(2)]
    
    stc03: Annotated[D_306, Title('Action Code'), Usage('N'), Position(3)]
    
    stc04: Annotated[D_782, Title('Monetary Amount'), Usage('N'), Position(4)]
    
    stc05: Annotated[D_782, Title('Monetary Amount'), Usage('N'), Position(5)]
    
    stc06: Annotated[D_373, Title('Date'), Usage('N'), Position(6)]
    
    stc07: Annotated[D_591, Title('Payment Method Code'), Usage('N'), Position(7)]
    
    stc08: Annotated[D_373, Title('Date'), Usage('N'), Position(8)]
    
    stc09: Annotated[D_429, Title('Check Number'), Usage('N'), Position(9)]
    
    stc10: Annotated[L2200C_STC10, Title('Health Care Claim Status'), Usage('S'), Position(10), Required(True)]
    
    stc11: Annotated[L2200C_STC11, Title('Health Care Claim Status'), Usage('S'), Position(11), Required(True)]
    
    stc12: Annotated[D_933, Title('Free-form Message Text'), Usage('N'), Position(12)]


class L2200C(Loop):
    
    trn: Annotated[L2200C_TRN, Title('Provider of Service Trace Identifier'), Usage('R'), Position(900), Required(True)]
    ItemStc: TypeAlias = Annotated[L2200C_STC, Title('Provider Status Information'), Usage('R'), Position(1000), Required(True)]
    stc: Annotated[list[ItemStc], MinItems(1)]


class L2000C(Loop):
    
    hl: Annotated[L2000C_HL, Title('Service Provider Level'), Usage('R'), Position(100), Required(True)]
    ItemL2100C: TypeAlias = Annotated[L2100C, Title('Provider Name'), Usage('R'), Position(500), Required(True)]
    l2100c: Annotated[list[ItemL2100C], MinItems(2)]
    ItemL2200C: TypeAlias = Annotated[L2200C, Title('Provider of Service Trace Identifier'), Usage('S'), Position(900), Required(True)]
    l2200c: Annotated[list[ItemL2200C], MinItems(1)]


class TABLE2AREA4(Loop):
    ItemL2000C: TypeAlias = Annotated[L2000C, Title('Service Provider Level'), Usage('S'), Position(100), Required(True)]
    l2000c: Annotated[list[ItemL2000C], MinItems(1)]


class L2000D_HL(Segment):
    """Subscriber Level"""
    _segment_name = 'HL'
    
    hl01: Annotated[D_628, Title('Hierarchical ID Number'), Usage('R'), Position(1)]
    
    hl02: Annotated[D_734, Title('Hierarchical Parent ID Number'), Usage('R'), Position(2)]
    
    hl03: Annotated[D_735, Title('Hierarchical Level Code'), Usage('R'), Position(3), Enumerated(*['22'])]
    
    hl04: Annotated[D_736, Title('Hierarchical Child Code'), Usage('R'), Position(4), Enumerated(*['0', '1'])]


class L2100D_NM1(Segment):
    """Subscriber Name"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title('Entity Identifier Code'), Usage('R'), Position(1), Enumerated(*['IL'])]
    
    nm102: Annotated[D_1065, Title('Entity Type Qualifier'), Usage('R'), Position(2), Enumerated(*['1', '2'])]
    
    nm103: Annotated[D_1035, Title('Subscriber Last Name'), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title('Subscriber First Name'), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title('Subscriber Middle Name or Initial'), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title('Name Prefix'), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title('Subscriber Name Suffix'), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title('Identification Code Qualifier'), Usage('R'), Position(8), Enumerated(*['24', 'II', 'MI'])]
    
    nm109: Annotated[D_67, Title('Subscriber Identifier'), Usage('R'), Position(9)]
    
    nm110: Annotated[D_706, Title('Entity Relationship Code'), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title('Entity Identifier Code'), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title('Name Last or Organization Name'), Usage('N'), Position(12)]


class L2100D(Loop):
    
    nm1: Annotated[L2100D_NM1, Title('Subscriber Name'), Usage('R'), Position(500), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]


class L2200D_TRN(Segment):
    """Claim Status Tracking Number"""
    _segment_name = 'TRN'
    
    trn01: Annotated[D_481, Title('Trace Type Code'), Usage('R'), Position(1), Enumerated(*['2'])]
    
    trn02: Annotated[D_127, Title('Referenced Transaction Trace Number'), Usage('R'), Position(2)]
    
    trn03: Annotated[D_509, Title('Originating Company Identifier'), Usage('N'), Position(3)]
    
    trn04: Annotated[D_127, Title('Reference Identification'), Usage('N'), Position(4)]


class L2200D_STC01(Composite):
    """Health Care Claim Status"""
    
    stc01_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc01_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc01_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*['03', '13', '17', '1E', '1G', '1H', '1I', '1O', '1P', '1Q', '1R', '1S', '1T', '1U', '1V', '1W', '1X', '1Y', '1Z', '28', '2A', '2B', '2D', '2E', '2I', '2K', '2P', '2Q', '2S', '2Z', '30', '36', '3A', '3C', '3D', '3E', '3F', '3G', '3H', '3I', '3J', '3K', '3L', '3M', '3N', '3O', '3P', '3Q', '3R', '3S', '3T', '3U', '3V', '3W', '3X', '3Y', '3Z', '40', '43', '44', '4A', '4B', '4C', '4D', '4E', '4F', '4G', '4H', '4I', '4J', '4L', '4M', '4N', '4O', '4P', '4Q', '4R', '4S', '4U', '4V', '4W', '4X', '4Y', '4Z', '5A', '5B', '5C', '5D', '5E', '5F', '5G', '5H', '5I', '5J', '5K', '5L', '5M', '5N', '5O', '5P', '5Q', '5R', '5S', '5T', '5U', '5V', '5W', '5X', '5Y', '5Z', '61', '6A', '6B', '6C', '6D', '6E', '6F', '6G', '6H', '6I', '6J', '6K', '6L', '6M', '6N', '6O', '6P', '6Q', '6R', '6S', '6U', '6V', '6W', '6X', '6Y', '71', '72', '73', '74', '77', '7C', '80', '82', '84', '85', '87', '95', 'CK', 'CZ', 'D2', 'DD', 'DJ', 'DK', 'DN', 'DO', 'DQ', 'E1', 'E2', 'E7', 'E9', 'FA', 'FD', 'FE', 'G0', 'G3', 'GB', 'GD', 'GI', 'GJ', 'GK', 'GM', 'GY', 'HF', 'HH', 'I3', 'IJ', 'IL', 'IN', 'LI', 'LR', 'MR', 'MSC', 'OB', 'OD', 'OX', 'P0', 'P2', 'P3', 'P4', 'P6', 'P7', 'PRP', 'PT', 'PV', 'PW', 'QA', 'QB', 'QC', 'QD', 'QE', 'QH', 'QK', 'QL', 'QN', 'QO', 'QS', 'QV', 'QY', 'RC', 'RW', 'S4', 'SEP', 'SJ', 'SU', 'T4', 'TL', 'TQ', 'TT', 'TTP', 'TU', 'UH', 'X3', 'X4', 'X5', 'ZZ'])]
    
    stc01_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('S'), Position(4), Enumerated(*['RX'])]


class L2200D_STC10(Composite):
    """Health Care Claim Status"""
    
    stc10_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc10_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc10_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3)]
    
    stc10_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('S'), Position(4), Enumerated(*['RX'])]


class L2200D_STC11(Composite):
    """Health Care Claim Status"""
    
    stc11_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc11_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc11_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3)]
    
    stc11_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('S'), Position(4), Enumerated(*['RX'])]


class L2200D_STC(Segment):
    """Claim Level Status Information"""
    _segment_name = 'STC'
    
    stc01: Annotated[L2200D_STC01, Title('Health Care Claim Status'), Usage('R'), Position(1), Required(True)]
    
    stc02: Annotated[D_373, Title('Status Information Effective Date'), Usage('R'), Position(2)]
    
    stc03: Annotated[D_306, Title('Action Code'), Usage('N'), Position(3)]
    
    stc04: Annotated[D_782, Title('Total Claim Charge Amount'), Usage('S'), Position(4)]
    
    stc05: Annotated[D_782, Title('Claim Payment Amount'), Usage('S'), Position(5)]
    
    stc06: Annotated[D_373, Title('Adjudication Finalized Date'), Usage('S'), Position(6)]
    
    stc07: Annotated[D_591, Title('Payment Method Code'), Usage('N'), Position(7)]
    
    stc08: Annotated[D_373, Title('Remittance Date'), Usage('S'), Position(8)]
    
    stc09: Annotated[D_429, Title('Remittance Trace Number'), Usage('S'), Position(9)]
    
    stc10: Annotated[L2200D_STC10, Title('Health Care Claim Status'), Usage('S'), Position(10), Required(True)]
    
    stc11: Annotated[L2200D_STC11, Title('Health Care Claim Status'), Usage('S'), Position(11), Required(True)]
    
    stc12: Annotated[D_933, Title('Free-form Message Text'), Usage('N'), Position(12)]


class L2200D_REF04(Composite):
    """Reference Identifier"""
    pass


class L2200D_REF(Segment):
    """Payer Claim Control Number"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['1K'])]
    
    ref02: Annotated[D_127, Title('Payer Claim Control Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200D_REF04(Composite):
    """Reference Identifier"""
    pass


class L2200D_REF(Segment):
    """Institutional Bill Type Identification"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['BLT'])]
    
    ref02: Annotated[D_127, Title('Bill Type Identifier'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200D_REF04(Composite):
    """Reference Identifier"""
    pass


class L2200D_REF(Segment):
    """Patient Control Number"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['EJ'])]
    
    ref02: Annotated[D_127, Title('Patient Control Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200D_REF04(Composite):
    """Reference Identifier"""
    pass


class L2200D_REF(Segment):
    """Pharmacy Prescription Number"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['XZ'])]
    
    ref02: Annotated[D_127, Title('Pharmacy Prescription Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200D_REF04(Composite):
    """Reference Identifier"""
    pass


class L2200D_REF(Segment):
    """Voucher Identifier"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['VV'])]
    
    ref02: Annotated[D_127, Title('Voucher Identifier'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200D_REF04(Composite):
    """Reference Identifier"""
    pass


class L2200D_REF(Segment):
    """Claim Identification Number For Clearinghouses and Other Transmission Intermediaries"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['D9'])]
    
    ref02: Annotated[D_127, Title('Clearinghouse Trace Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200D_DTP(Segment):
    """Claim Service Date"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title('Date Time Qualifier'), Usage('R'), Position(1), Enumerated(*['472'])]
    
    dtp02: Annotated[D_1250, Title('Date Time Period Format Qualifier'), Usage('R'), Position(2), Enumerated(*['D8', 'RD8'])]
    
    dtp03: Annotated[D_1251, Title('Claim Service Period'), Usage('R'), Position(3)]


class L2220D_SVC01(Composite):
    """Composite Medical Procedure Identifier"""
    
    svc01_01: Annotated[D_235, Title('Product or Service ID Qualifier'), Usage('R'), Position(1), Enumerated(*['AD', 'ER', 'HC', 'HP', 'IV', 'N4', 'NU', 'WK'])]
    
    svc01_02: Annotated[D_234, Title('Procedure Code'), Usage('R'), Position(2)]
    
    svc01_03: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(3)]
    
    svc01_04: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(4)]
    
    svc01_05: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(5)]
    
    svc01_06: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(6)]
    
    svc01_07: Annotated[D_352, Title('Description'), Usage('N'), Position(7)]
    
    svc01_08: Annotated[D_234, Title('Product/Service ID'), Usage('N'), Position(8)]


class L2220D_SVC06(Composite):
    """Composite Medical Procedure Identifier"""
    pass


class L2220D_SVC(Segment):
    """Service Line Information"""
    _segment_name = 'SVC'
    
    svc01: Annotated[L2220D_SVC01, Title('Composite Medical Procedure Identifier'), Usage('R'), Position(1), Required(True)]
    
    svc02: Annotated[D_782, Title('Line Item Charge Amount'), Usage('R'), Position(2)]
    
    svc03: Annotated[D_782, Title('Line Item Payment Amount'), Usage('R'), Position(3)]
    
    svc04: Annotated[D_234, Title('Revenue Code'), Usage('S'), Position(4)]
    
    svc05: Annotated[D_380, Title('Quantity'), Usage('N'), Position(5)]
    
    svc07: Annotated[D_380, Title('Units of Service Count'), Usage('R'), Position(7)]


class L2220D_STC01(Composite):
    """Health Care Claim Status"""
    
    stc01_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc01_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc01_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*['03', '13', '17', '1E', '1G', '1H', '1I', '1O', '1P', '1Q', '1R', '1S', '1T', '1U', '1V', '1W', '1X', '1Y', '1Z', '28', '2A', '2B', '2D', '2E', '2I', '2K', '2P', '2Q', '2S', '2Z', '30', '36', '3A', '3C', '3D', '3E', '3F', '3G', '3H', '3I', '3J', '3K', '3L', '3M', '3N', '3O', '3P', '3Q', '3R', '3S', '3T', '3U', '3V', '3W', '3X', '3Y', '3Z', '40', '43', '44', '4A', '4B', '4C', '4D', '4E', '4F', '4G', '4H', '4I', '4J', '4L', '4M', '4N', '4O', '4P', '4Q', '4R', '4S', '4U', '4V', '4W', '4X', '4Y', '4Z', '5A', '5B', '5C', '5D', '5E', '5F', '5G', '5H', '5I', '5J', '5K', '5L', '5M', '5N', '5O', '5P', '5Q', '5R', '5S', '5T', '5U', '5V', '5W', '5X', '5Y', '5Z', '61', '6A', '6B', '6C', '6D', '6E', '6F', '6G', '6H', '6I', '6J', '6K', '6L', '6M', '6N', '6O', '6P', '6Q', '6R', '6S', '6U', '6V', '6W', '6X', '6Y', '71', '72', '73', '74', '77', '7C', '80', '82', '84', '85', '87', '95', 'CK', 'CZ', 'D2', 'DD', 'DJ', 'DK', 'DN', 'DO', 'DQ', 'E1', 'E2', 'E7', 'E9', 'FA', 'FD', 'FE', 'G0', 'G3', 'GB', 'GD', 'GI', 'GJ', 'GK', 'GM', 'GY', 'HF', 'HH', 'I3', 'IJ', 'IL', 'IN', 'LI', 'LR', 'MR', 'MSC', 'OB', 'OD', 'OX', 'P0', 'P2', 'P3', 'P4', 'P6', 'P7', 'PRP', 'PT', 'PV', 'PW', 'QA', 'QB', 'QC', 'QD', 'QE', 'QH', 'QK', 'QL', 'QN', 'QO', 'QS', 'QV', 'QY', 'RC', 'RW', 'S4', 'SEP', 'SJ', 'SU', 'T4', 'TL', 'TQ', 'TT', 'TTP', 'TU', 'UH', 'X3', 'X4', 'X5', 'ZZ'])]
    
    stc01_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('S'), Position(4), Enumerated(*['RX'])]


class L2220D_STC10(Composite):
    """Health Care Claim Status"""
    
    stc10_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc10_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc10_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3)]
    
    stc10_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('S'), Position(4), Enumerated(*['RX'])]


class L2220D_STC11(Composite):
    """Health Care Claim Status"""
    
    stc11_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc11_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc11_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3)]
    
    stc11_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('S'), Position(4), Enumerated(*['RX'])]


class L2220D_STC(Segment):
    """Service Line Status Information"""
    _segment_name = 'STC'
    
    stc01: Annotated[L2220D_STC01, Title('Health Care Claim Status'), Usage('R'), Position(1), Required(True)]
    
    stc02: Annotated[D_373, Title('Status Information Effective Date'), Usage('R'), Position(2)]
    
    stc03: Annotated[D_306, Title('Action Code'), Usage('N'), Position(3)]
    
    stc04: Annotated[D_782, Title('Monetary Amount'), Usage('N'), Position(4)]
    
    stc05: Annotated[D_782, Title('Monetary Amount'), Usage('N'), Position(5)]
    
    stc06: Annotated[D_373, Title('Date'), Usage('N'), Position(6)]
    
    stc07: Annotated[D_591, Title('Payment Method Code'), Usage('N'), Position(7)]
    
    stc08: Annotated[D_373, Title('Date'), Usage('N'), Position(8)]
    
    stc09: Annotated[D_429, Title('Check Number'), Usage('N'), Position(9)]
    
    stc10: Annotated[L2220D_STC10, Title('Health Care Claim Status'), Usage('S'), Position(10), Required(True)]
    
    stc11: Annotated[L2220D_STC11, Title('Health Care Claim Status'), Usage('S'), Position(11), Required(True)]
    
    stc12: Annotated[D_933, Title('Free-form Message Text'), Usage('N'), Position(12)]


class L2220D_REF04(Composite):
    """Reference Identifier"""
    pass


class L2220D_REF(Segment):
    """Service Line Item Identification"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['FJ'])]
    
    ref02: Annotated[D_127, Title('Line Item Control Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2220D_DTP(Segment):
    """Service Line Date"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title('Date Time Qualifier'), Usage('R'), Position(1), Enumerated(*['472'])]
    
    dtp02: Annotated[D_1250, Title('Date Time Period Format Qualifier'), Usage('R'), Position(2), Enumerated(*['D8', 'RD8'])]
    
    dtp03: Annotated[D_1251, Title('Service Line Date'), Usage('R'), Position(3)]


class L2220D(Loop):
    
    svc: Annotated[L2220D_SVC, Title('Service Line Information'), Usage('R'), Position(1800), Required(True)]
    ItemStc: TypeAlias = Annotated[L2220D_STC, Title('Service Line Status Information'), Usage('R'), Position(1900), Required(True)]
    stc: Annotated[list[ItemStc], MinItems(1)]
    
    ref: Annotated[L2220D_REF, Title('Service Line Item Identification'), Usage('S'), Position(2000), Syntax(['R0203']), Required(True)]
    
    dtp: Annotated[L2220D_DTP, Title('Service Line Date'), Usage('R'), Position(2100), Required(True)]


class L2200D(Loop):
    
    trn: Annotated[L2200D_TRN, Title('Claim Status Tracking Number'), Usage('R'), Position(900), Required(True)]
    ItemStc: TypeAlias = Annotated[L2200D_STC, Title('Claim Level Status Information'), Usage('R'), Position(1000), Required(True)]
    stc: Annotated[list[ItemStc], MinItems(1)]
    
    ref: Annotated[L2200D_REF, Title('Payer Claim Control Number'), Usage('S'), Position(1100), Syntax(['R0203']), Required(True)]
    
    ref: Annotated[L2200D_REF, Title('Institutional Bill Type Identification'), Usage('S'), Position(1100), Syntax(['R0203']), Required(True)]
    
    ref: Annotated[L2200D_REF, Title('Patient Control Number'), Usage('S'), Position(1100), Syntax(['R0203']), Required(True)]
    
    ref: Annotated[L2200D_REF, Title('Pharmacy Prescription Number'), Usage('S'), Position(1100), Syntax(['R0203']), Required(True)]
    
    ref: Annotated[L2200D_REF, Title('Voucher Identifier'), Usage('S'), Position(1100), Syntax(['R0203']), Required(True)]
    
    ref: Annotated[L2200D_REF, Title('Claim Identification Number For Clearinghouses and Other Transmission Intermediaries'), Usage('S'), Position(1100), Syntax(['R0203']), Required(True)]
    
    dtp: Annotated[L2200D_DTP, Title('Claim Service Date'), Usage('S'), Position(1200), Required(True)]
    ItemL2220D: TypeAlias = Annotated[L2220D, Title('Service Line Information'), Usage('S'), Position(1800), Required(True)]
    l2220d: Annotated[list[ItemL2220D], MinItems(1)]


class L2000D(Loop):
    
    hl: Annotated[L2000D_HL, Title('Subscriber Level'), Usage('R'), Position(100), Required(True)]
    ItemL2100D: TypeAlias = Annotated[L2100D, Title('Subscriber Name'), Usage('R'), Position(500), Required(True)]
    l2100d: Annotated[list[ItemL2100D], MinItems(1)]
    ItemL2200D: TypeAlias = Annotated[L2200D, Title('Claim Status Tracking Number'), Usage('S'), Position(900), Required(True)]
    l2200d: Annotated[list[ItemL2200D], MinItems(1)]


class TABLE2AREA5(Loop):
    ItemL2000D: TypeAlias = Annotated[L2000D, Title('Subscriber Level'), Usage('S'), Position(100), Required(True)]
    l2000d: Annotated[list[ItemL2000D], MinItems(1)]


class L2000E_HL(Segment):
    """Dependent Level"""
    _segment_name = 'HL'
    
    hl01: Annotated[D_628, Title('Hierarchical ID Number'), Usage('R'), Position(1)]
    
    hl02: Annotated[D_734, Title('Hierarchical Parent ID Number'), Usage('R'), Position(2)]
    
    hl03: Annotated[D_735, Title('Hierarchical Level Code'), Usage('R'), Position(3), Enumerated(*['23'])]
    
    hl04: Annotated[D_736, Title('Hierarchical Child Code'), Usage('N'), Position(4)]


class L2100E_NM1(Segment):
    """Dependent Name"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title('Entity Identifier Code'), Usage('R'), Position(1), Enumerated(*['QC'])]
    
    nm102: Annotated[D_1065, Title('Entity Type Qualifier'), Usage('R'), Position(2), Enumerated(*['1'])]
    
    nm103: Annotated[D_1035, Title('Patient Last Name'), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title('Patient First Name'), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title('Patient Middle Name or Initial'), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title('Name Prefix'), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title('Patient Name Suffix'), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title('Identification Code Qualifier'), Usage('N'), Position(8)]
    
    nm109: Annotated[D_67, Title('Identification Code'), Usage('N'), Position(9)]
    
    nm110: Annotated[D_706, Title('Entity Relationship Code'), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title('Entity Identifier Code'), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title('Name Last or Organization Name'), Usage('N'), Position(12)]


class L2100E(Loop):
    
    nm1: Annotated[L2100E_NM1, Title('Dependent Name'), Usage('R'), Position(500), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]


class L2200E_TRN(Segment):
    """Claim Status Tracking Number"""
    _segment_name = 'TRN'
    
    trn01: Annotated[D_481, Title('Trace Type Code'), Usage('R'), Position(1), Enumerated(*['2'])]
    
    trn02: Annotated[D_127, Title('Referenced Transaction Trace Number'), Usage('R'), Position(2)]
    
    trn03: Annotated[D_509, Title('Originating Company Identifier'), Usage('N'), Position(3)]
    
    trn04: Annotated[D_127, Title('Reference Identification'), Usage('N'), Position(4)]


class L2200E_STC01(Composite):
    """Health Care Claim Status"""
    
    stc01_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc01_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc01_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*['03', '13', '17', '1E', '1G', '1H', '1I', '1O', '1P', '1Q', '1R', '1S', '1T', '1U', '1V', '1W', '1X', '1Y', '1Z', '28', '2A', '2B', '2D', '2E', '2I', '2K', '2P', '2Q', '2S', '2Z', '30', '36', '3A', '3C', '3D', '3E', '3F', '3G', '3H', '3I', '3J', '3K', '3L', '3M', '3N', '3O', '3P', '3Q', '3R', '3S', '3T', '3U', '3V', '3W', '3X', '3Y', '3Z', '40', '43', '44', '4A', '4B', '4C', '4D', '4E', '4F', '4G', '4H', '4I', '4J', '4L', '4M', '4N', '4O', '4P', '4Q', '4R', '4S', '4U', '4V', '4W', '4X', '4Y', '4Z', '5A', '5B', '5C', '5D', '5E', '5F', '5G', '5H', '5I', '5J', '5K', '5L', '5M', '5N', '5O', '5P', '5Q', '5R', '5S', '5T', '5U', '5V', '5W', '5X', '5Y', '5Z', '61', '6A', '6B', '6C', '6D', '6E', '6F', '6G', '6H', '6I', '6J', '6K', '6L', '6M', '6N', '6O', '6P', '6Q', '6R', '6S', '6U', '6V', '6W', '6X', '6Y', '71', '72', '73', '74', '77', '7C', '80', '82', '84', '85', '87', '95', 'CK', 'CZ', 'D2', 'DD', 'DJ', 'DK', 'DN', 'DO', 'DQ', 'E1', 'E2', 'E7', 'E9', 'FA', 'FD', 'FE', 'G0', 'G3', 'GB', 'GD', 'GI', 'GJ', 'GK', 'GM', 'GY', 'HF', 'HH', 'I3', 'IJ', 'IL', 'IN', 'LI', 'LR', 'MR', 'MSC', 'OB', 'OD', 'OX', 'P0', 'P2', 'P3', 'P4', 'P6', 'P7', 'PRP', 'PT', 'PV', 'PW', 'QA', 'QB', 'QC', 'QD', 'QE', 'QH', 'QK', 'QL', 'QN', 'QO', 'QS', 'QV', 'QY', 'RC', 'RW', 'S4', 'SEP', 'SJ', 'SU', 'T4', 'TL', 'TQ', 'TT', 'TTP', 'TU', 'UH', 'X3', 'X4', 'X5', 'ZZ'])]
    
    stc01_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('S'), Position(4), Enumerated(*['RX'])]


class L2200E_STC10(Composite):
    """Health Care Claim Status"""
    
    stc10_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc10_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc10_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3)]
    
    stc10_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('S'), Position(4), Enumerated(*['RX'])]


class L2200E_STC11(Composite):
    """Health Care Claim Status"""
    
    stc11_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc11_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc11_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3)]
    
    stc11_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('S'), Position(4), Enumerated(*['RX'])]


class L2200E_STC(Segment):
    """Claim Level Status Information"""
    _segment_name = 'STC'
    
    stc01: Annotated[L2200E_STC01, Title('Health Care Claim Status'), Usage('R'), Position(1), Required(True)]
    
    stc02: Annotated[D_373, Title('Status Information Effective Date'), Usage('R'), Position(2)]
    
    stc03: Annotated[D_306, Title('Action Code'), Usage('N'), Position(3)]
    
    stc04: Annotated[D_782, Title('Total Claim Charge Amount'), Usage('S'), Position(4)]
    
    stc05: Annotated[D_782, Title('Claim Payment Amount'), Usage('S'), Position(5)]
    
    stc06: Annotated[D_373, Title('Adjudication Finalized Date'), Usage('S'), Position(6)]
    
    stc07: Annotated[D_591, Title('Payment Method Code'), Usage('N'), Position(7)]
    
    stc08: Annotated[D_373, Title('Remittance Date'), Usage('S'), Position(8)]
    
    stc09: Annotated[D_429, Title('Remittance Trace Number'), Usage('S'), Position(9)]
    
    stc10: Annotated[L2200E_STC10, Title('Health Care Claim Status'), Usage('S'), Position(10), Required(True)]
    
    stc11: Annotated[L2200E_STC11, Title('Health Care Claim Status'), Usage('S'), Position(11), Required(True)]
    
    stc12: Annotated[D_933, Title('Free-form Message Text'), Usage('N'), Position(12)]


class L2200E_REF04(Composite):
    """Reference Identifier"""
    pass


class L2200E_REF(Segment):
    """Payer Claim Control Number"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['1K'])]
    
    ref02: Annotated[D_127, Title('Payer Claim Control Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200E_REF04(Composite):
    """Reference Identifier"""
    pass


class L2200E_REF(Segment):
    """Institutional Bill Type Identification"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['BLT'])]
    
    ref02: Annotated[D_127, Title('Bill Type Identifier'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200E_REF04(Composite):
    """Reference Identifier"""
    pass


class L2200E_REF(Segment):
    """Patient Control Number"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['EJ'])]
    
    ref02: Annotated[D_127, Title('Patient Control Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200E_REF04(Composite):
    """Reference Identifier"""
    pass


class L2200E_REF(Segment):
    """Pharmacy Prescription Number"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['XZ'])]
    
    ref02: Annotated[D_127, Title('Pharmacy Prescription Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200E_REF04(Composite):
    """Reference Identifier"""
    pass


class L2200E_REF(Segment):
    """Voucher Identifier"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['VV'])]
    
    ref02: Annotated[D_127, Title('Voucher Identifier'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200E_REF04(Composite):
    """Reference Identifier"""
    pass


class L2200E_REF(Segment):
    """Claim Identification Number For Clearinghouses and Other Transmission Intermediaries"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['D9'])]
    
    ref02: Annotated[D_127, Title('Clearinghouse Trace Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200E_DTP(Segment):
    """Claim Service Date"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title('Date Time Qualifier'), Usage('R'), Position(1), Enumerated(*['472'])]
    
    dtp02: Annotated[D_1250, Title('Date Time Period Format Qualifier'), Usage('R'), Position(2), Enumerated(*['D8', 'RD8'])]
    
    dtp03: Annotated[D_1251, Title('Claim Service Period'), Usage('R'), Position(3)]


class L2220E_SVC01(Composite):
    """Composite Medical Procedure Identifier"""
    
    svc01_01: Annotated[D_235, Title('Product or Service ID Qualifier'), Usage('R'), Position(1), Enumerated(*['AD', 'ER', 'HC', 'HP', 'IV', 'N4', 'NU', 'WK'])]
    
    svc01_02: Annotated[D_234, Title('Procedure Code'), Usage('R'), Position(2)]
    
    svc01_03: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(3)]
    
    svc01_04: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(4)]
    
    svc01_05: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(5)]
    
    svc01_06: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(6)]
    
    svc01_07: Annotated[D_352, Title('Description'), Usage('N'), Position(7)]
    
    svc01_08: Annotated[D_234, Title('Product/Service ID'), Usage('N'), Position(8)]


class L2220E_SVC06(Composite):
    """Composite Medical Procedure Identifier"""
    pass


class L2220E_SVC(Segment):
    """Service Line Information"""
    _segment_name = 'SVC'
    
    svc01: Annotated[L2220E_SVC01, Title('Composite Medical Procedure Identifier'), Usage('R'), Position(1), Required(True)]
    
    svc02: Annotated[D_782, Title('Line Item Charge Amount'), Usage('R'), Position(2)]
    
    svc03: Annotated[D_782, Title('Line Item Payment Amount'), Usage('R'), Position(3)]
    
    svc04: Annotated[D_234, Title('Revenue Code'), Usage('S'), Position(4)]
    
    svc05: Annotated[D_380, Title('Quantity'), Usage('N'), Position(5)]
    
    svc07: Annotated[D_380, Title('Units of Service Count'), Usage('R'), Position(7)]


class L2220E_STC01(Composite):
    """Health Care Claim Status"""
    
    stc01_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc01_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc01_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*['03', '13', '17', '1E', '1G', '1H', '1I', '1O', '1P', '1Q', '1R', '1S', '1T', '1U', '1V', '1W', '1X', '1Y', '1Z', '28', '2A', '2B', '2D', '2E', '2I', '2K', '2P', '2Q', '2S', '2Z', '30', '36', '3A', '3C', '3D', '3E', '3F', '3G', '3H', '3I', '3J', '3K', '3L', '3M', '3N', '3O', '3P', '3Q', '3R', '3S', '3T', '3U', '3V', '3W', '3X', '3Y', '3Z', '40', '43', '44', '4A', '4B', '4C', '4D', '4E', '4F', '4G', '4H', '4I', '4J', '4L', '4M', '4N', '4O', '4P', '4Q', '4R', '4S', '4U', '4V', '4W', '4X', '4Y', '4Z', '5A', '5B', '5C', '5D', '5E', '5F', '5G', '5H', '5I', '5J', '5K', '5L', '5M', '5N', '5O', '5P', '5Q', '5R', '5S', '5T', '5U', '5V', '5W', '5X', '5Y', '5Z', '61', '6A', '6B', '6C', '6D', '6E', '6F', '6G', '6H', '6I', '6J', '6K', '6L', '6M', '6N', '6O', '6P', '6Q', '6R', '6S', '6U', '6V', '6W', '6X', '6Y', '71', '72', '73', '74', '77', '7C', '80', '82', '84', '85', '87', '95', 'CK', 'CZ', 'D2', 'DD', 'DJ', 'DK', 'DN', 'DO', 'DQ', 'E1', 'E2', 'E7', 'E9', 'FA', 'FD', 'FE', 'G0', 'G3', 'GB', 'GD', 'GI', 'GJ', 'GK', 'GM', 'GY', 'HF', 'HH', 'I3', 'IJ', 'IL', 'IN', 'LI', 'LR', 'MR', 'MSC', 'OB', 'OD', 'OX', 'P0', 'P2', 'P3', 'P4', 'P6', 'P7', 'PRP', 'PT', 'PV', 'PW', 'QA', 'QB', 'QC', 'QD', 'QE', 'QH', 'QK', 'QL', 'QN', 'QO', 'QS', 'QV', 'QY', 'RC', 'RW', 'S4', 'SEP', 'SJ', 'SU', 'T4', 'TL', 'TQ', 'TT', 'TTP', 'TU', 'UH', 'X3', 'X4', 'X5', 'ZZ'])]
    
    stc01_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('S'), Position(4), Enumerated(*['RX'])]


class L2220E_STC10(Composite):
    """Health Care Claim Status"""
    
    stc10_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc10_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc10_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3)]
    
    stc10_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('S'), Position(4), Enumerated(*['RX'])]


class L2220E_STC11(Composite):
    """Health Care Claim Status"""
    
    stc11_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1)]
    
    stc11_02: Annotated[D_1271, Title('Status Code'), Usage('R'), Position(2)]
    
    stc11_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3)]
    
    stc11_04: Annotated[D_1270, Title('Code List Qualifier Code'), Usage('S'), Position(4), Enumerated(*['RX'])]


class L2220E_STC(Segment):
    """Service Line Status Information"""
    _segment_name = 'STC'
    
    stc01: Annotated[L2220E_STC01, Title('Health Care Claim Status'), Usage('R'), Position(1), Required(True)]
    
    stc02: Annotated[D_373, Title('Status Information Effective Date'), Usage('R'), Position(2)]
    
    stc03: Annotated[D_306, Title('Action Code'), Usage('N'), Position(3)]
    
    stc04: Annotated[D_782, Title('Monetary Amount'), Usage('N'), Position(4)]
    
    stc05: Annotated[D_782, Title('Monetary Amount'), Usage('N'), Position(5)]
    
    stc06: Annotated[D_373, Title('Date'), Usage('N'), Position(6)]
    
    stc07: Annotated[D_591, Title('Payment Method Code'), Usage('N'), Position(7)]
    
    stc08: Annotated[D_373, Title('Date'), Usage('N'), Position(8)]
    
    stc09: Annotated[D_429, Title('Check Number'), Usage('N'), Position(9)]
    
    stc10: Annotated[L2220E_STC10, Title('Health Care Claim Status'), Usage('S'), Position(10), Required(True)]
    
    stc11: Annotated[L2220E_STC11, Title('Health Care Claim Status'), Usage('S'), Position(11), Required(True)]
    
    stc12: Annotated[D_933, Title('Free-form Message Text'), Usage('N'), Position(12)]


class L2220E_REF04(Composite):
    """Reference Identifier"""
    pass


class L2220E_REF(Segment):
    """Service Line Item Identification"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['FJ'])]
    
    ref02: Annotated[D_127, Title('Line Item Control Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2220E_DTP(Segment):
    """Service Line Date"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title('Date Time Qualifier'), Usage('R'), Position(1), Enumerated(*['472'])]
    
    dtp02: Annotated[D_1250, Title('Date Time Period Format Qualifier'), Usage('R'), Position(2), Enumerated(*['D8', 'RD8'])]
    
    dtp03: Annotated[D_1251, Title('Service Line Date'), Usage('R'), Position(3)]


class L2220E(Loop):
    
    svc: Annotated[L2220E_SVC, Title('Service Line Information'), Usage('R'), Position(1800), Required(True)]
    ItemStc: TypeAlias = Annotated[L2220E_STC, Title('Service Line Status Information'), Usage('R'), Position(1900), Required(True)]
    stc: Annotated[list[ItemStc], MinItems(1)]
    
    ref: Annotated[L2220E_REF, Title('Service Line Item Identification'), Usage('S'), Position(2000), Syntax(['R0203']), Required(True)]
    
    dtp: Annotated[L2220E_DTP, Title('Service Line Date'), Usage('R'), Position(2100), Required(True)]


class L2200E(Loop):
    
    trn: Annotated[L2200E_TRN, Title('Claim Status Tracking Number'), Usage('R'), Position(900), Required(True)]
    ItemStc: TypeAlias = Annotated[L2200E_STC, Title('Claim Level Status Information'), Usage('R'), Position(1000), Required(True)]
    stc: Annotated[list[ItemStc], MinItems(1)]
    
    ref: Annotated[L2200E_REF, Title('Payer Claim Control Number'), Usage('S'), Position(1100), Syntax(['R0203']), Required(True)]
    
    ref: Annotated[L2200E_REF, Title('Institutional Bill Type Identification'), Usage('S'), Position(1100), Syntax(['R0203']), Required(True)]
    
    ref: Annotated[L2200E_REF, Title('Patient Control Number'), Usage('S'), Position(1100), Syntax(['R0203']), Required(True)]
    
    ref: Annotated[L2200E_REF, Title('Pharmacy Prescription Number'), Usage('S'), Position(1100), Syntax(['R0203']), Required(True)]
    
    ref: Annotated[L2200E_REF, Title('Voucher Identifier'), Usage('S'), Position(1100), Syntax(['R0203']), Required(True)]
    
    ref: Annotated[L2200E_REF, Title('Claim Identification Number For Clearinghouses and Other Transmission Intermediaries'), Usage('S'), Position(1100), Syntax(['R0203']), Required(True)]
    
    dtp: Annotated[L2200E_DTP, Title('Claim Service Date'), Usage('S'), Position(1200), Required(True)]
    ItemL2220E: TypeAlias = Annotated[L2220E, Title('Service Line Information'), Usage('S'), Position(1800), Required(True)]
    l2220e: Annotated[list[ItemL2220E], MinItems(1)]


class L2000E(Loop):
    
    hl: Annotated[L2000E_HL, Title('Dependent Level'), Usage('R'), Position(100), Required(True)]
    ItemL2100E: TypeAlias = Annotated[L2100E, Title('Dependent Name'), Usage('R'), Position(500), Required(True)]
    l2100e: Annotated[list[ItemL2100E], MinItems(1)]
    ItemL2200E: TypeAlias = Annotated[L2200E, Title('Claim Status Tracking Number'), Usage('R'), Position(900), Required(True)]
    l2200e: Annotated[list[ItemL2200E], MinItems(1)]


class TABLE2AREA6(Loop):
    ItemL2000E: TypeAlias = Annotated[L2000E, Title('Dependent Level'), Usage('S'), Position(100), Required(True)]
    l2000e: Annotated[list[ItemL2000E], MinItems(1)]


class DETAIL(Loop):
    ItemL2000A: TypeAlias = Annotated[L2000A, Title('Information Source Level'), Usage('R'), Position(100), Required(True)]
    l2000a: Annotated[list[ItemL2000A], MinItems(1)]
    ItemTable2Area4: TypeAlias = Annotated[TABLE2AREA4, Title('Table2 - Area4'), Usage('S'), Position(140)]
    table2area4: Annotated[list[ItemTable2Area4], MinItems(1)]
    ItemTable2Area5: TypeAlias = Annotated[TABLE2AREA5, Title('Table2 - Area5'), Usage('S'), Position(150)]
    table2area5: Annotated[list[ItemTable2Area5], MinItems(1)]
    ItemTable2Area6: TypeAlias = Annotated[TABLE2AREA6, Title('Table2 - Area6'), Usage('S'), Position(160)]
    table2area6: Annotated[list[ItemTable2Area6], MinItems(1)]


class ST_LOOP(Loop):
    
    st: Annotated[ST_LOOP_ST, Title('Transaction Set Header'), Usage('R'), Position(100), Required(True)]
    ItemHeader: TypeAlias = Annotated[HEADER, Title('Header'), Usage('R'), Position(110), Required(True)]
    header: Annotated[list[ItemHeader], MinItems(1)]
    ItemDetail: TypeAlias = Annotated[DETAIL, Title('Table2 - Detail'), Usage('R'), Position(120), Required(True)]
    detail: Annotated[list[ItemDetail], MinItems(1)]


class FOOTER(Loop):
    pass


class TABLE2AREA3_SE(Segment):
    """Transaction Set Trailer"""
    _segment_name = 'SE'
    
    se01: Annotated[D_96, Title('Transaction Segment Count'), Usage('R'), Position(1)]
    
    se02: Annotated[D_329, Title('Transaction Set Control Number'), Usage('R'), Position(2)]


class TABLE2AREA3(Loop):
    ItemFooter: TypeAlias = Annotated[FOOTER, Title('Footer'), Usage('N'), Position(170)]
    footer: Annotated[list[ItemFooter], MinItems(1)]
    
    se: Annotated[TABLE2AREA3_SE, Title('Transaction Set Trailer'), Usage('R'), Position(2700), Required(True)]


class GS_LOOP_GE(Segment):
    """Functional Group Trailer"""
    _segment_name = 'GE'
    
    ge01: Annotated[D_97, Title('Number of Transaction Sets Included'), Usage('R'), Position(1)]
    
    ge02: Annotated[D_28, Title('Group Control Number'), Usage('R'), Position(2)]


class GS_LOOP(Loop):
    
    gs: Annotated[GS_LOOP_GS, Title('Functional Group Header'), Usage('R'), Position(100), Required(True)]
    ItemSt_Loop: TypeAlias = Annotated[ST_LOOP, Title('Transaction Set Header'), Usage('R'), Position(200), Required(True)]
    st_loop: Annotated[list[ItemSt_Loop], MinItems(1)]
    
    table2area3: Annotated[TABLE2AREA3, Title('Table2 - Area3'), Usage('R'), Position(130), Required(True)]
    
    ge: Annotated[GS_LOOP_GE, Title('Functional Group Trailer'), Usage('R'), Position(300), Required(True)]


class ISA_LOOP_TA1(Segment):
    """Interchange Acknowledgement"""
    _segment_name = 'TA1'
    
    ta101: Annotated[I12, Title('Interchange Control Number'), Usage('R'), Position(1)]
    
    ta102: Annotated[I08, Title('Interchange Date'), Usage('R'), Position(2)]
    
    ta103: Annotated[I09, Title('Interchange Time'), Usage('R'), Position(3)]
    
    ta104: Annotated[I17, Title('Interchange Acknowledgement Code'), Usage('R'), Position(4), Enumerated(*['A', 'E', 'R'])]
    
    ta105: Annotated[I18, Title('Interchange Note Code'), Usage('R'), Position(5), Enumerated(*['000', '001', '002', '003', '004', '005', '006', '007', '008', '009', '010', '011', '012', '013', '014', '015', '016', '017', '018', '019', '020', '021', '022', '023', '024', '025', '026', '027', '028', '029', '030', '031'])]


class ISA_LOOP_IEA(Segment):
    """Interchange Control Trailer"""
    _segment_name = 'IEA'
    
    iea01: Annotated[I16, Title('Number of Included Functional Groups'), Usage('R'), Position(1)]
    
    iea02: Annotated[I12, Title('Interchange Control Number'), Usage('R'), Position(2)]


class ISA_LOOP(Loop):
    
    isa: Annotated[ISA_LOOP_ISA, Title('Interchange Control Header'), Usage('R'), Position(100), Required(True)]
    ItemGs_Loop: TypeAlias = Annotated[GS_LOOP, Title('Functional Group Header'), Usage('R'), Position(200), Required(True)]
    gs_loop: Annotated[list[ItemGs_Loop], MinItems(1)]
    
    ta1: Annotated[ISA_LOOP_TA1, Title('Interchange Acknowledgement'), Usage('S'), Position(200), Required(True)]
    
    iea: Annotated[ISA_LOOP_IEA, Title('Interchange Control Trailer'), Usage('R'), Position(300), Required(True)]


class MSG277A1(Message):
    """HIPAA Health Care Information Status Notification 005010X212 277A1"""
    ItemIsa_Loop: TypeAlias = Annotated[ISA_LOOP, Title('Interchange Control Header'), Usage('R'), Position(10), Required(True)]
    isa_loop: Annotated[list[ItemIsa_Loop], MinItems(1)]



"""
Base definitions for X12 message constructs:

- Element -- the atomic base value -- A python object -- str, int, float, Decimal

- Composite -- a sequence of Elements

- Segment -- a sequence of [Composite | Element] with a prefix code (e.g., "ISA", "IEA")

- Loop -- a sequence of [Loop | Segment]

- Message -- a sequence of Loop

The :py:class:`Source` class defines a parser
for the message in "Exchange" format.

-   Elements have an element separator (often "|")

-   Composites have a component separator (varies widely, but ":" is common)

-   Segments have a segment separator (often "~")

The various X12 structures are subclasses of :py:class:`Message`,
:py:class:`Loop`, :py:class:`Segment`, and :py:class:`Composite`.
The details of each element are implemented as ``Annotated[...]``
type hints.

Parsing
=======

The top-level (Message, Loop, Segment) structures all have a :meth:`parse` method
to parse a :meth:`Source` instance. This consumes whole segments.

The bottom-level (Composite) structure has a :meth:`build` method that
consumes one or more field values from a parsed segment.

JSON Schema
===========

The :py:func:`schema` function will
emit a class definition as JSON Schema.
This can be used with :func:`json.dumps` to present
a JSON dump of a schema.

JSON Output
===========

The :py:func:`asdict` function will
emit an object as a dictionary.
This can be used with :func:`json.dumps` to present
a JSON dump of a message.

The :py:meth:`Message.json` method does this
for a message.

Message Segments
================

The :py:meth:`Message.segment_iter` method
yields the sequence of segments.
This is similar to exchange format for a message.

"""
from collections import defaultdict
from collections.abc import Iterator, Callable, Hashable
import datetime
from decimal import Decimal
import fnmatch
from functools import wraps, cache
import json
import logging
import sys
from textwrap import dedent
from types import GenericAlias, UnionType, NoneType, FunctionType
from typing import (  # type: ignore [attr-defined]
    Any, cast, Protocol, overload,
    Union, TypeAlias, Annotated, Optional, DefaultDict,
    get_type_hints, get_origin, get_args, assert_type,
    _SpecialForm,
    _UnionGenericAlias,
)
from typing_extensions import _AnnotatedAlias
import warnings

from .annotations import *

logger = logging.getLogger("x12.base")


class Source:
    """
    The Source class consumes the text used to parse a message in "Exchange format."

    High-level client's :meth:`parse` method will consume an entire segment, up to the segment separator.
    This applies to :class:`Method`, :class:`Loop`, and :class:`Segment`.
    The :meth:`next_segment` method lets a client peek at the prefix to the next segment.

    Low-level client's :meth:`build` method will consume element values from a segment.
    The :meth:`elements` method decomposes the current segment, and advances to the next segment.

    State invariant: ``self.text[self.pos]`` is the start of a segment.

    When parsing the initial ISA segment, there's no defined ``element_sep``.
    During parsing the ISA, the ``element_sep`` and ``segment_sep`` can be located,
    and used for all following segments. The ``array_sep`` (or component separator) is explicitly
    provided in the ISA segment; this is used to decompose composites.

    In some rare cases, the ISA is already compressed. The element and segment
    separation characters must be provided separately.

    ..  todo:: Consider str | TextIO | Path as ``text`` parameter types.
    """
    logger = logging.getLogger("x12.base.Source")  #: Logger for this class

    def __init__(self, text: str, element_sep: str = "", segment_sep: str = "", array_sep: str = "") -> None:
        """
        Build a Source to be used for parsing a message.

        :param text: the Exchange-format document
        :param element_sep: Optional element separator character. This can be deduced by parsing the ISA segment.
        :param segment_sep: Optional segment separator character. This can be deduced by parsing the ISA segment.
        :param array_sep: Optional component separator character. This is generally found in the ISA16 element of the ISA segment.
        """
        self.text = text
        self.element_sep = element_sep  # Often "|", might be "*".
        self.segment_sep = segment_sep  # Often "~".
        # The Component_Separator is the value of ISA16. It is not, generally, set in advance.
        self.array_sep = array_sep  # Often ":". Sometimes "^".
        self.pos: int = 0

    def consume_whitespace(self) -> None:
        """
        Moves ``self.pos`` past any leading whitespace.
        """
        while self.pos != len(self.text) and self.text[self.pos].isspace():
            self.pos += 1

    def elements(self) -> list[str | list[str]]:
        """
        Unpack a Segment into a list of potential Element values.
        This consumes whitespace.
        It consumes the segment up to (and including) the segment_sep character.

        Note the initial ISA segment is fixed length, not compressed. Separators aren't defined yet.
        The initial ISA segment's text is returned as a single string for parsing by the segment.

        Most elements are atomic with two notable exceptions:

        -   There array array-like elements that leverage
            the ISA16 Component Separator character, called the
            ``array_sep`` here.

        -   Composites are punctuated by the ISA16 Component Separator character.

        We choose to parse these non-atomic elements here by looking at the values
        and splitting any that have the separator character.

        Conceptually:

            >>> source = "field|comp:osite|field~"
            >>> segments = source.split("~")
            >>> fields = segments[0].split("|")
            >>> fields = [v.split(":") if ":" in v else v for v in fields]
            >>> fields
            ['field', ['comp', 'osite'], 'field']

        ..  todo:: Optimize the field and composite splitting to avoid creating multiple lists.

        ..  todo:: re.search() might be faster at finding segment separators than the ``while`` loop.
        """
        if not self.segment_sep:
            raise RuntimeError("no element or segment separators set")  # pragma: no cover
        self.consume_whitespace()
        # TODO: re.search() might be faster at finding the next segment separator
        start = self.pos
        while self.pos != len(self.text) and self.text[self.pos] != self.segment_sep:
            self.pos += 1
        # assert self.text[self.pos] == self.segment_sep
        if self.pos != len(self.text):
            self.pos += 1
        elements = self.text[start:self.pos-1].rstrip().split(self.element_sep)
        # ISA segment? Do NOT subdivide ISA16 field's value -- it would vanish into ``["", ""]``.
        # TODO: Alternative design is for ``Segment`` ISA parser can replace ISA16 ``["", ""]`` value with the parsed array_sep value.
        if elements[0].upper() == "ISA":
            return cast(list[str | list[str]], elements)
        return [
            val.split(self.array_sep) if self.array_sep in val else val
            for val in elements
        ]

    def peek(self, size: int = 106) -> str:
        """
        Peek ahead a fixed number of characters. This also consumes leading whitespace.
        The default peek is 106 to read a non-compressed ISA segment.
        """
        self.consume_whitespace()
        # Handle the weird case of segment separator **preceeded** by whitespace.
        end = self.pos + size
        segment = self.text[self.pos: end].rstrip()
        while (
                len(segment) != size and
                end != len(self.text)
        ):
            self.logger.debug("Whitespace before segment separator")
            if not self.text[end].isspace():
                segment += self.text[end]
            end += 1
        return segment

    def next_segment(self) -> str:
        """Peek at upcoming segment's header. This consumes leading whitespace."""
        self.consume_whitespace()
        if not self.element_sep:
            # Assume a 3-char prefix until we have a separator
            return self.text[self.pos: self.pos+3]
        # TODO: re.search() might be faster at finding the next element separator
        start = end = self.pos
        while end != len(self.text) and self.text[end] != self.element_sep:
            end += 1
        return self.text[start: end]

    def __repr__(self) -> str:
        return f"{self.element_sep=} {self.segment_sep=} text={self.text[:self.pos]!r} && {self.text[self.pos:]!r}"

class ConversionCallable(Protocol):
    """A protocol for verious kinds of conversions."""
    @overload
    def __call__(self, value: str, format: str) -> Any: ...
    @overload
    def __call__(self, value: str, scale: int) -> Any: ...
    @overload
    def __call__(self, value: str | None = None, extra: str | None = None) -> Any: ...

Conversion: TypeAlias = ConversionCallable | type[Any]

Formatter: TypeAlias = Callable[["X12ElementHelper", Any], str]


class X12ElementHelper:
    """
    An X12ElementHelper
    provides methos to convert element values from source text
    to Python objects, and Python back to source text.

    Don't create an :py:mod:`X12ElementHelper` directly. Use the :py:meth:`X12ElementHelper.annotated` method.

    For example::

        hints = get_type_hints(SomeSegment, include_extras=True)
        some_helper = base.X12ElementHelper.annotated(hints['some_element'])


    The :py:meth:`X12ElementHelper.validate` method raises a :exc:`ValueError`
    exception  with two parameters:

    -   The usual message.

    -   The Annotation class name that failed validation.

    ..  TODO: A hierarchy of classes can handle primitive formatting methods better.

        The "format" feature should be subclasses not method plug-ins.
    """
    @staticmethod
    def _date(source: str, format: str) -> Any:
        return datetime.datetime.strptime(source, format).date()

    @staticmethod
    def _time(source: str, format: str) -> Any:
        return datetime.datetime.strptime(source, format).time()

    @classmethod
    def annotated(cls, type_hint: type[Any]) -> Union["X12ElementHelper", None]:
        """
        Build an :py:class:`X12ElementHelper` instance to convert an element's text to Python objects,
        and python objects back to text.

        This does *not* handle List[X] and Union[X, None].
        """
        # Wrapped classes: list[X], X | None, Annotated[X, ...]
        base: type[Any]
        aspects: list[X12Annotation]
        match type_hint:
            case UnionType() | _UnionGenericAlias():  # type: ignore[misc]
                return None
            case GenericAlias() if get_origin(type_hint) is list:
                return None
            case _AnnotatedAlias():  # type: ignore[misc]
                # An "Annotated[X]" -- extract the annotation aspects
                base, *aspects = get_args(type_hint)
                # Continues in second match step, below.
            case type():
                # A bare "X" -- work with what we have
                base = type_hint
                aspects = []
                # Continues in a second match step, below.
            case _:  # pragma: no cover
                raise TypeError(f"invalid {type_hint}")  # pragma: no cover

        # For Annotated[X] or bare X, create a helper
        # These don't seem to work well with ``match``.
        if issubclass(base, datetime.date):
            # TODO: Make this sensisitve to MinLen() and MaxLen()
            return X12ElementHelper(
                cast(Conversion, cls._date),
                *aspects,
                parse_formats=["%y%m%d", "%Y%m%d", ],
                formatter=cls.dttm_fmt
            )
        elif issubclass(base, datetime.time):
            # TODO: Make this sensisitve to MinLen() and MaxLen()
            return X12ElementHelper(
                cast(Conversion, cls._time),
                *aspects,
                parse_formats=["%H%M", "%H%M%S", ],
                formatter=cls.dttm_fmt
            )
        else:
            match base():
                case Composite():  # type() if issubclass(base, Composite):
                    return X12ElementHelper(cast(Conversion, lambda x: x), *aspects)
                case str():  # type() if base == str:
                    return X12ElementHelper(cast(Conversion, str), *aspects, formatter=cls.str_fmt)
                case int():  # type() if base == int:
                    return X12ElementHelper(cast(Conversion, int), *aspects, formatter=cls.int_fmt)
                case float():  # type() if base == float:
                    return X12ElementHelper(cast(Conversion, float), *aspects, formatter=cls.float_fmt)
                case Decimal():  # type() if base == Decimal:
                    scale_aspect = [a for a in aspects if isinstance(a, Scale)]
                    if scale_aspect:
                        scale = int(scale_aspect[0].params[0])
                    else:  # pragma: no cover
                        scale = 0
                    # Bind scale into a decimal conversion function
                    decimal_conversion_partial: Conversion = cast(
                            Conversion,
                            lambda to_decimal=None, scale=scale: (Decimal(to_decimal) if to_decimal else Decimal()).scaleb(-scale)
                    )
                    return X12ElementHelper(
                        decimal_conversion_partial,
                        *aspects,
                        formatter=cls.dec_fmt
                    )

        # Silently tolerate other structures.
        return None

    def __init__(self, conversion: Conversion, *aspects: X12Annotation, parse_formats: list[str] | None = None, formatter: Formatter | None = None) -> None:
        """
        Build an X12ElementHelper.

        :param conversion: The conversion from text to Python function.
        :param aspects: The X12Annotations from the Annotated type.
        :param parse_formats: A sequence of formats for parsing dates and times.
        :param formatter: The conversion from Python back to text. :py:func:`str` is the default.
        """
        self.conversion = conversion  # the to-python conversion function
        self.formatter = formatter or cast(Formatter, str)  # the to-text formatter function
        self.aspects = aspects  # The annotations that will be checked.
        # TODO: Seems to be a silly optimization.
        self.validation = [
            a
            for a in self.aspects
            if a.validation
        ]  # Subset of annotations that can be validated.
        self.optional = (
            any(
                a.params[0] in {"N", "S"} for a in self.aspects
                if isinstance(a, (Usage,))
            ) or any(
                not a.params[0] for a in self.aspects
                if isinstance(a, (Required,))
            )
        )  # Interpret X12 rules to see if this is optional
        self.parse_formats: list[str] = parse_formats or []  # Alternative date/time formats.
        self.dttm_format: str | None = None  # Set to the preferred cormat for dates or times.
        self.max_len = self._value_for(MaxLen) or 99
        self.min_len = self._value_for(MinLen) or 0
        self.scale = self._value_for(Scale)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}({self.conversion}, *{self.aspects}, parse_formats={self.parse_formats or None}, formatter={self.formatter})"

    def _value_for(self, annotation_class: type[X12Annotation]) -> Any:
        """Find the last value set for a specific annotation class."""
        instances = [a for a in self.aspects if isinstance(a, annotation_class)]
        if instances:
            return instances[-1].params[0]
        else:
            return None

    def validate(self, source: str) -> None:
        if self.optional and (source == '' or source is None):
            return
        for a in self.validation:
            if msg := a.invalid(source):
                raise ValueError(msg, a.__class__.__name__)

    def to_py(self, source: str) -> Any:
        if source is None:
            # TODO: Check ``Usage`` and ``Required`` annotations
            return None
        conv_func = self.conversion
        if self.parse_formats:
            # Date/Time with multiple formats.
            for fmt in self.parse_formats:
                try:
                    v = conv_func(source, fmt)
                    self.dttm_format = fmt
                    return v
                except ValueError:
                    pass
            raise ValueError(f"invalid {source!r} formats {self.parse_formats}")  # pragma: no cover
        else:
            # Other types without format alternatives.
            if source == "":
                # Assume the conversion is a type, str, int, float, Decimal
                try:
                    return cast(type[Any], conv_func())
                except TypeError as ex:  # pragma: no cover
                    # Rare issue with conversion function being used improperly.
                    raise TypeError(f"{conv_func} {self.aspects} {ex}")
            else:
                return conv_func(source)

    def to_str(self, value: Any) -> str:
        # print(f"in {self!r}, to_str({value=}) with {self.formatter=}")
        return self.formatter(self, value)

    def str_fmt(self, value: Any) -> str:
        return f"{value:{self.min_len}s}"

    def int_fmt(self, value: Any) -> str:
        return f"{value:0>{self.min_len}d}"

    def float_fmt(self, value: Any) -> str:
        return f"{value:0>{self.min_len}.0f}"

    def dec_fmt(self, value: Any) -> str:
        base_text = f"{value:.{self.scale}f}".replace(".", "")
        return f"{base_text:0>{self.min_len}s}"

    def dttm_fmt(self, value: Any) -> str:
        if not self.dttm_format:
            self.dttm_format = self.parse_formats[0]
        return f"{value:{self.dttm_format}}"


SegmentText: TypeAlias = list[str | None | list[str | None]]


def schema(some_type: type[Any]) -> dict[str, Any]:
    """
    Expands an annotated type or any of the X12 structures into a JSON Schema.

    This is very similar to the pydantic function :py:func:`schema_of`.

    :param some_type: Any kind of type. A subclass of :py:class:`Message` is typical.
    :returns: JSON Schema.
    """

    def x12_class_base_schema(some_type: type[Any]) -> dict[str, Any]:
        properties = {
            name: schema(field_type)
            for name, field_type in class_fields(some_type)
        }
        json_schema = {
            "description": dedent(some_type.__doc__.strip()) if some_type.__doc__ else None,
            "class_name": some_type.__name__ if isinstance(some_type, type) else some_type.__class__.__name__,
            "type": "object",
            "properties": properties,
        }
        return json_schema

    annotations: list[X12Annotation] = []
    match some_type:
        case GenericAlias() if get_origin(some_type) is list:
            repeating_type, *annotations = get_args(some_type)
            json_schema = {
                "type": "array",
                "items": schema(repeating_type),
            }
        case _AnnotatedAlias():  # type: ignore [misc]
            base_type, *annotations = get_args(some_type)
            json_schema = schema(base_type)
        case UnionType() | _UnionGenericAlias():  # type: ignore[misc]
            alternatives = get_args(some_type)
            json_schema = {
                "any": [schema(alt) for alt in alternatives]
            }
        case _:
            if issubclass(some_type, datetime.date):
                # TODO: X12ElementHelper has formats.
                json_schema = {"type": "string", "convert": "datetime.date", "pyformat": "%Y%m%d"}
            elif issubclass(some_type, datetime.time):
                json_schema = {"type": "string", "convert": "datetime.time", "pyformat": "%h%m"}
            else:
                match some_type():
                    case Composite() | Segment() | Loop() | Message():
                        json_schema = x12_class_base_schema(some_type)
                    # Classes with built-in primitive schema.
                    case str():
                        json_schema = {"type": "string"}
                    case int():
                        json_schema = {"type": "integer"}
                    case float():
                        json_schema = {"type": "number"}
                    case Decimal():
                        json_schema = {"type": "string", "convert": "Decimal", "scale": "?"}
                    case None:
                        json_schema = {"type": "null"}
                    case _:  # pragma: no cover
                        raise ValueError(f"can't make schema for {some_type} ({type(some_type)})")
    # Effectively a reduce(set.union, (ann.JSONSchema for ann in annotations if hasattr(ann, "JSONSchema")))
    for ann in annotations:
        if hasattr(ann, "JSONSchema"):
            json_schema |= ann.JSONSchema
    return json_schema

X12Structure: TypeAlias = Union[
    "Composite", "Segment", "Loop", "Message",
    str, int, float, Decimal, datetime.date, datetime.time,
    None
]

def asdict(x12_obj: X12Structure) -> dict[str, Any] | Any:
    """
    Given an X12 object (i.e., Message, Loop, Segment, Composite), emit a dictionary.

    :param x12_obj: Any X12 structure.
    :returns: a dictionary, suitable for JSON output.
    """
    match x12_obj:
        case Composite() | Segment() | Loop() | Message():
            fields = {
                name: getattr(x12_obj, name, None)
                for name, hint in class_fields(x12_obj.__class__)
            }
            non_null_fields = {
                name: asdict(value)
                for name, value in fields.items()
                if value is not None
            }
            parent = x12_obj.__class__
            return {
                '_kind': x12_obj.__class__.__mro__[1].__name__,
                '_name': x12_obj.__class__.__name__,
            } | non_null_fields
        case str() | int() | float():
            return x12_obj
        case Decimal():
            return str(x12_obj)
        case datetime.date():
            format = "%Y%m%d"
            return {"date": x12_obj.strftime(format), "_format": format}
        case datetime.time():
            format = "%H%M"
            return {"time": x12_obj.strftime(format), "_format": format}
        case list(some_items):
            return [
                asdict(item) for item in some_items
            ]
        case _:  # pragma: no cover
            raise ValueError(f"can't create value for {x12_obj} ({type(x12_obj)})")


class FieldInspector:
    """
    A ``FieldInspector`` walks the definition of fields in an X12 structure,
    providing those not excluded by a few rules:

    -   Excludes "Schema" as a special attribute name.

    -   Excludes all names starting with "_".

    -   Excludes any ``TypeAlias`` lines in the class.

    The result of scanning a structure is a list of (name, hint) tuples.

    The :py:func:`class_fields` function is the useful instance of this class.
    """
    def __init__(self) -> None:
        self.cache: dict[type[Any], list[tuple[str, type[Any]]]] = {}

    def field_iter(self, cls: type[Any]) -> Iterator[tuple[str, type[Any]]]:
        fields = get_type_hints(cls, include_extras=True)
        # Exclude "Schema" and names starting with _
        x_schema_hidden = (
            (name, field_type)
            for name, field_type in fields.items()
            if not (name in {"Schema"} or name.startswith("_"))
        )
        # Exclude the TypeAlias special form.
        x_special = (
            (name, field_type)
            for name, field_type in x_schema_hidden
            if not (isinstance(field_type, _SpecialForm) and field_type._name == "TypeAlias")  # type: ignore[attr-defined]
        )
        return x_special

    def field_list(self, cls: type[Any]) -> list[tuple[str, type[Any]]]:
        if cls not in self.cache:
            self.cache[cls] = list(self.field_iter(cls))
        return self.cache[cls]

cached_field_inspector = FieldInspector()

class_fields = cached_field_inspector.field_list  #: Function to use a :py:class:`FieldInspector` on a class, returns all non-excluded fields.


class Composite:
    """
    A Composite is a collection of elements.

    In the exchange format, they're separated the "array_sep".
    For example, the ``array_sep=":"`` and
    The :py:class:`Source` may have text ``value|comp:osite|value``.
    The composite will be parsed into ``["comp", "osite"]``

    Composites are often defined as follows.

    ::

        class Loop_Seg_Comp(Composite):
            field_2_1: Annotated[...]
            field_2_2: Annotated[...]
            etc.

        class Loop_Seg(Segment):
            field_1: Annotated[...]
            field_2: Loop_Seg_Comp
            etc.

    A Composite is generally built using the :py:meth:`composite.build` method and sequence of values.
    """
    _composite_name: str  #: Name of the composite
    _helpers: dict[str, X12ElementHelper | None]  #: X12ElementHelpers for element conversion and validation
    _skip_validation: DefaultDict[str, list[str]]  #: Elements not to be validated
    # No type hint: invisible to get_type_hints() introspection
    logger = logging.getLogger("x12.base.Composite")  #: Class Logger

    def __init__(self, **arg_dict) -> None:
        """
        Create a Composite instance by providing keyword argument values
        for each field.

        This will validate each value provided.
        """
        fields = get_type_hints(self.__class__, include_extras=True)
        unknowns = arg_dict.keys() - fields.keys()
        if unknowns:
            raise TypeError(f"unknown fields: {unknowns}")  # pragma: no cover
        self.__class__.make_helpers(fields)

        # Ideally, all missing names are also not required.
        for name, hint in class_fields(self.__class__):
            source: str = cast(str, arg_dict.get(name))
            value: Any
            if self._helpers[name] is not None:
                helper: X12ElementHelper = cast(X12ElementHelper, self._helpers[name])
                # Validate (possibly raising ValueError)
                try:
                    helper.validate(source)
                except ValueError as ex:
                    if not (name in self._skip_validation and len(ex.args) == 2):
                        raise ValueError(f"invalid {self.__class__.__name__}.{name}: {hint} {ex}") from ex
                    msg, annotation = ex.args
                    if annotation not in self._skip_validation[name]:
                        raise ValueError(f"invalid {self.__class__.__name__}.{name}: {hint} {ex}") from ex
                # Build the attribute value
                value = helper.to_py(source)
            else:
                value = source
            setattr(self, name, value)

    @classmethod
    def make_helpers(cls, fields: dict[str, type[Any]]) -> None:
        """
        Build the element ``_helpers`` for this Composite.

        Used like this::

            class XYZ(Composite):
                x01: float
                x02: float

            fields = get_type_hints(XYZ, include_extras=True)
            XYZ.make_helpers(fields)

        This updates the class with detailed helpers for each field.
        """
        if hasattr(cls, '_helpers'):
            return

        # Assume it contains *only* elements.
        cls._helpers = {
            name: X12ElementHelper.annotated(hint)
            for name, hint in class_fields(cls)
        }

    @classmethod
    def composite_configure(cls, skip_validation : DefaultDict[str, list[str]], segment_rules: list[tuple[str, str]], composite_name: str) -> None:
        """
        Provide the skip_validation configuration to the elements of this Composite.
        """
        cls._skip_validation = skip_validation
        fields = get_type_hints(cls, include_extras=True)
        cls.make_helpers(fields)
        for name, helper in cls._helpers.items():
            for field_match, annotation_name in segment_rules:
                if fnmatch.fnmatch(name, field_match):
                    cls._skip_validation[name].append(annotation_name)

    def elements(self) -> list[str | None | list[str | None | list[str | None]]]:
        """
        Returns the list of values for this composite.

        ..  note:: Confusing overlap of this elements() method with :py:class:`Source.elements` method.

            This is **not** the same concept at all.
        """
        fields = get_type_hints(self)
        return [getattr(self, name) for name, type_hint in class_fields(self.__class__)]

    def __repr__(self) -> str:
        good_names = (name for name, field_type in class_fields(self.__class__))
        text = ", ".join(f"{name}={getattr(self, name)!r}" for name in good_names)
        return f"{self.__class__.__name__}({text})"


class Segment:
    """
    A Segment is a collection of ``Elements | Composite``.

    Segments have the element_sep separators;
    any contained composites use the array_sep separtors.

    The lexical scanner turns these into sub-lists of the element value list.
    So ``seg|element1|comp:osite|element3`` will become the following list of
    values: ``["seg", "element1", ["comp", "osite"], "element3"]``.

    The ISA Segment is special. It's generally fixed-length, and it defines the element and segment separators
    used everywhere else.

    Segments are defined as a list of elements.

    ::

        class Loop_Segment(Segment):
            field_1: Annotated[str, MinLen(2), MaxLen(2)]
            field_2: Annotated[Decimal, Scale(2)]
            etc.

    When parsing the ISA segment, we handle this as a distinct special case.

    ..   note:: IndexError: pop from empty list

        often means wrong separators.
    """
    _segment_name: str  #: String name to match to parse this segment
    _helpers: dict[str, X12ElementHelper | None]  #: X12ElementHelper for field conversion and validation
    _skip_validation: DefaultDict[str, list[str]]  #: Elements not to be validated
    # No type hint: invisible to get_type_hints() introspection
    logger = logging.getLogger("x12.base.Segment")  #: Class Logger

    def __init__(self, **arg_dict) -> None:
        """
        Create a Segment instance by providing keyword argument values
        for each field.

        This will validate each value provided.
        """
        fields = get_type_hints(self.__class__, include_extras=True)
        unknowns = arg_dict.keys() - fields.keys()
        if unknowns:
            raise TypeError(f"unknown fields: {unknowns}")  # pragma: no cover
        self.__class__.make_helpers(fields)

        # Ideally all missing names are also not required.
        for name, hint in class_fields(self.__class__):
            ## TODO: Replace `None` with a "do nothing" X12ElementHelper instance.
            source: str = cast(str, arg_dict.get(name))
            value: Any
            if self._helpers[name] is not None:
                helper: X12ElementHelper = cast(X12ElementHelper, self._helpers[name])
                # Validate (possibly raising ValueError)
                try:
                    helper.validate(source)
                except ValueError as ex:
                    if not (name in self._skip_validation and len(ex.args) == 2):
                        # print(f"{self.__class__.__name__} {self._skip_validation=} {name=} {ex.args=}")
                        raise ValueError(f"invalid {self.__class__.__name__}.{name}: {hint} {ex}") from ex
                    msg, annotation = ex.args
                    if annotation not in self._skip_validation[name]:
                        # print(f"{self.__class__.__name__} {self._skip_validation=}")
                        raise ValueError(f"invalid {self.__class__.__name__}.{name}: {hint} {ex}") from ex
                # Convert
                value = helper.to_py(source)
            else:
                # No helper? Use the value without further ado (it's likely a Composite or a list[X])
                value = source
            setattr(self, name, value)

    @classmethod
    def make_helpers(cls, fields: dict[str, type[Any]]) -> None:
        """
        Build the element ``_helpers`` for this Segment.

        Used like this:

        ::

            fields = get_type_hints(SomeSegment, include_extras=True)
            SomeSegment.make_helpers(fields)
        """
        if hasattr(cls, '_helpers'):
            return
        cls._helpers = {}
        for name, hint in class_fields(cls):
            cls._helpers[name] = cls._make_helper(name, hint)

    @classmethod
    def _make_helper(cls, name: str, hint: type[Any]) -> X12ElementHelper | None:
        """
        This class creates helper wrappers for List[X] and Union[X, None].
        Otherwise, use :py:class:`X12ElementHelper` for primitives.
        """
        match hint:
            case UnionType() | _UnionGenericAlias():  # type: ignore[misc]
                # # TODO: Create a X | Y | None to_py construct.
                # # If a validate() works, that's the result, None.
                # # If none of the validate()'s work, and there's a None, that's it.
                # # Otherwise, a ValueError exception from a failing validate.

                alternatives = list(filter(lambda t: t is not NoneType, get_args(hint)))
                if len(alternatives) != 1:
                    raise NotImplementedError(
                        f"not supported {hint}; too many {alternatives}")  # pragma: no cover
                base = alternatives[0]
                # TODO: make X12UnionHelper.annotated(base); return X12UnionHelper(hint) instance
                # Instead, we quietly ignore the Union.
                return cls._make_helper(name, base)
            case GenericAlias() if get_origin(hint) == list:
                repeating, *aspects = get_args(hint)
                item_helper = X12ElementHelper.annotated(repeating)
                # TODO: Create A "list[X]" to_py that checks MaxItems (and MinItems)
                list_conversion: Conversion = cast(
                    Conversion,
                    lambda some_list: [item_helper.to_py(v) for v in some_list]
                )
                return X12ElementHelper(list_conversion, *aspects)
            case _AnnotatedAlias():  # type: ignore[misc]
                # Annotated. Create a helper
                helper = X12ElementHelper.annotated(hint)
                if helper is None:
                    # Something more elaborate than a primitive.
                    # usually Annotated[list[Annotated[base, etc.], etc.]
                    # base is list[Annotated]
                    base, *annotations = get_args(hint)
                    # TODO: Build AnnotatedListHelper.
                    return cls._make_helper(name, base)
                else:
                    return helper
            case type() if issubclass(hint, Composite):
                hint.make_helpers(get_type_hints(hint, include_extras=True))
                # TODO: Return an X12CompositeHelper() that does nothing.
                return None
            case type():
                # Primitive
                return X12ElementHelper.annotated(hint)
        raise ValueError(f"unexpected {name}: {hint} {type(hint)=} {get_origin(hint)=} in {cls}")  # pragma: no cover

    @classmethod
    def configure(cls, skip_validation : list[str]) -> None:
        """
        Provide the skip_validation configuration to the elements of this Segment.
        """
        fields = get_type_hints(cls, include_extras=True)
        cls.make_helpers(fields)
        all_rules = (
            rule.split(":")
            for rule in skip_validation
        )
        try:
            segment_rules = [
                (field_match, annotation_name)
                for class_match, field_match, annotation_name in all_rules
                if fnmatch.fnmatch(cls.__name__, class_match)
            ]
        except ValueError:  # pragma: no cover
            raise ValueError(f"invalid skip_validation rule in {skip_validation}")
        cls.logger.debug("Skip Validation: %s", segment_rules)

        cls._skip_validation = defaultdict(list)

        for field_match, annotation_name in segment_rules:
            # Does this match any names in the class?
            # print(f"configure {field_match=} {annotation_name=} in {cls.__name__}")
            for name, helper in cls._helpers.items():
                if fnmatch.fnmatch(name, field_match):
                    cls._skip_validation[name].append(annotation_name)

        # Propogate down to composites.
        for name, field_type in class_fields(cls):
            # TODO: PEEL THE ONION
            while type(field_type) is not type:
                match field_type:
                    case _AnnotatedAlias():  # type: ignore[misc]
                        # Annotated[f, ...]
                        field_type = get_args(field_type)[0]
                    case UnionType() | _UnionGenericAlias():  # type: ignore[misc]
                        # f | None
                        alternatives = list(filter(lambda t: t is not NoneType, get_args(field_type)))
                        if len(alternatives) != 1:
                            raise NotImplementedError(
                                f"not supported {field_type}; too many {alternatives}")  # pragma: no cover
                        field_type = alternatives[0]
                    case GenericAlias() if get_origin(field_type) is list:
                        # List[f]
                        field_type = get_args(field_type)[0]
                    case _:  # pragma: no cover
                        raise ValueError(f"unexpected {name}: {field_type} {type(field_type)=} {get_origin(field_type)=} in {cls}")
            # print(f"configure {field_type}, {type(field_type)}")
            if issubclass(field_type, Composite):
                field_type.composite_configure(cls._skip_validation, segment_rules, name)

    def elements(self) -> SegmentText:
        """
        The segment as a list of Element source values.
        The segment text is the basis for the exchange format: a sequence of Segments.

        ..  note:: Confusing overlap of this elements() method with :py:class:`Source.elements` method.

            This is **not** the same concept at all.

        ..  todo:: Rename this to not conflict with semantics of `Source`.
        """
        values: SegmentText = []
        for name, hint in class_fields(self.__class__):
            field: Composite | list[Composite] = getattr(self, name)
            match field:
                case list():
                    values.append(cast(list[str | None], [v.elements() for v in field]))
                case Composite():
                    values.append(cast(list[str | None], field.elements()))
                case _:
                    # Annotated fields
                    self.logger.debug(f"{name}: {field} {self._helpers[name]} {getattr(self, name)=}")
                    values.append(getattr(self, name))
        return values

    def __repr__(self) -> str:
        good_names = (name for name, hint in class_fields(self.__class__))
        text = ", ".join(f"{name}={getattr(self, name)!r}" for name in good_names)
        return f"{self.__class__.__name__}({text})"


class Loop:
    """
    A Loop is a collection of ``Loop | Segment`` instances.

    Loops (and their Segments) can repeat, or may be optional.

    We need to examine the loop's structure of segments
    and sub-loops my matching segment prefixes to see
    which segment can be parsed.
    """
    logger = logging.getLogger("x12.base.Loop")  #: Class Logger

    def __init__(self, **arg_dict) -> None:
        """
        Create a Loop instance by providing keyword argument values
        for each segment or nested loop.

        The Segments and Composites will validate any values provided.
        """
        fields = get_type_hints(self.__class__)
        unknowns = arg_dict.keys() - fields.keys()
        if unknowns:
            raise TypeError(f"unknown fields: {unknowns}")  # pragma: no cover
        # Ideally all missing names are also not required.
        for name, hint in class_fields(self.__class__):
            setattr(self, name, arg_dict.get(name))

    @classmethod
    def configure(cls, skip_validation : list[str]) -> None:
        """
        Provide the skip_validation configuration to the Segments of this Loop.
        """
        for name, field_type in class_fields(cls):
            # TODO: PEEL THE ONION...
            while type(field_type) is not type:
                match field_type:
                    case GenericAlias() if get_origin(field_type) is list:
                        # list[X]
                        field_type = get_args(field_type)[0]
                    case UnionType() | _UnionGenericAlias():  # type: ignore[misc]
                        # Something | None OR list[Something] | None
                        alternatives = list(filter(lambda t: t is not NoneType, get_args(field_type)))
                        field_type = alternatives[0]
                    case _AnnotatedAlias():  # type: ignore[misc]
                        field_type = get_args(field_type)[0]
                    case _:  # pragma: no cover
                        raise ValueError(f"unexpected {name}: {field_type} {type(field_type)=} {get_origin(field_type)=} in {cls}")
            if issubclass(field_type, Segment):
                field_type.configure(skip_validation)
            elif issubclass(field_type, Loop):
                field_type.configure(skip_validation)
            else:
                raise TypeError(f"unexpected {name}: {field_type} {type(field_type)}")  # pragma: no cover

    def segment_iter(self) -> Iterator[SegmentText]:
        """
        A flat sequence of segment values in this Loop and all sub Loops.
        This is the basis for the exchange format.
        """
        for name, hint in class_fields(self.__class__):
            field = getattr(self, name)
            self.logger.debug("segment_iter() name=%s field=%s", name, field)
            match field:
                case list():  # list[Something]
                    for instance in field:
                        if isinstance(instance, Loop):
                            yield from instance.segment_iter()
                        elif isinstance(instance, Segment):
                            name = cast(Segment, instance)._segment_name
                            yield cast(SegmentText, [name]) + instance.elements()
                        else:
                            raise TypeError(f"unexpected {name}: {hint} instance {instance=}")  # pragma: no cover
                case Loop(): # _ if isinstance(field, Loop):
                    # This doesn't seem to arise in practice.
                    yield from field.segment_iter()  # pragma: no cover
                case Segment(): # _ if isinstance(field, Segment):
                    name = cast(Segment, field)._segment_name
                    yield cast(SegmentText, [name]) + field.elements()
                case None:
                    # TypeAlias will have a value of None.
                    pass
                case _:  # pragma: no cover
                    raise TypeError(f"unexpected {name}: {field}")

    def __repr__(self) -> str:
        good_names = (n for n, h in class_fields(self.__class__))
        text = ", ".join(f"{name}={getattr(self, name)}" for name in good_names if hasattr(self, name))
        return f"{self.__class__.__name__}({text})"


class Message:
    """
    A top-level message, a collection of Loop instances.

    Loops can (and often do) repeat.
    """
    logger = logging.getLogger("x12.base.Message")  #: Class Logger

    def __init__(self, **arg_dict) -> None:
        """
        Create a Message instance by providing keyword argument values
        for each Loop.

        The Segments and Composites will validate any values provided.
        """
        fields = get_type_hints(self.__class__)
        unknowns = arg_dict.keys() - fields.keys()
        if unknowns:
            raise TypeError(f"unknown fields: {unknowns}")  # pragma: no cover
        # Ideally all missing names are also not required.
        for name, hint in class_fields(self.__class__):
            setattr(self, name, arg_dict.get(name))

    @classmethod
    def configure(cls, skip_validation : list[str]) -> None:
        """
        Provide the skip_validation configuration to the Segments of this Loop.
        """
        for name, loop_type in class_fields(cls):
            # TODO: PEEL THE ONION.
            while type(loop_type) is not type:
                match loop_type:
                    case GenericAlias() if get_origin(loop_type) is list:  # type: ignore[misc]
                        loop_type = get_args(loop_type)[0]
                    case UnionType() | _UnionGenericAlias():  # type: ignore[misc]
                        # Something | None OR list[Something] | None
                        alternatives = list(filter(lambda t: t is not NoneType, get_args(loop_type)))
                        loop_type = alternatives[0]
                    case _AnnotatedAlias():  # type: ignore[misc]
                        loop_type = get_args(loop_type)[0]
                    case _:  # pragma: no cover
                        raise TypeError(f"unexpected {name}: {loop_type} {type(loop_type)}")  # pragma: no cover
            if issubclass(loop_type, Loop):
                loop_type.configure(skip_validation)
            else:
                raise TypeError(f"unexpected {name}: {loop_type} {type(loop_type)}")  # pragma: no cover

    def segment_iter(self) -> Iterator[list[str | None | list[str | None]]]:
        """
        A flat sequence of segments in a Message.
        This is the basis for the exchange format.
        """
        for name, hint in class_fields(self.__class__):
            loop = getattr(self, name)
            match loop:
                case list():
                    for instance in loop:
                        yield from instance.segment_iter()
                case Loop():
                    yield from loop.segment_iter()
                case _:  # pragma: no cover
                    raise TypeError(f"unexpected {name}: {hint} {loop=}")

    def __repr__(self) -> str:
        good_names = (n for n, h in class_fields(self.__class__))
        text = ", ".join(f"{name}={getattr(self, name)}" for name in good_names)
        return f"{self.__class__.__name__}({text})"

    def __bool__(self):
        return any(getattr(self, name) for name, hint in class_fields(self.__class__))

    def json(self, indent: int | None = None) -> str:
        return json.dumps(asdict(self), indent=indent)

    def asdict(self) -> dict[str, Any]:
        return asdict(self)


class X12Parser:
    """
    Walks the structure of the annotated components in a given class.
    parses the Segment text blocks.

    This handles the various annotation cases supported by X12 messages:

    -   Optional[_T] == Union[_T, None].

    -   list[_T]

    -   Annotated[_T, ...]

    -   _T

    It also honors the nesting structure of messages, loops, segments, composites, and elements.

    """
    def __init__(self, message_class: type[Message], skip_validation : list[str] | None = None) -> None:
        self.cls = message_class
        self.cls.configure(skip_validation or [])
        self.logger = logging.getLogger(self.__class__.__name__)

    def parse(self, source: Source) -> Message:
        """
        Parses the Exchange format text for this Message subclass.

        Locates all Loops; each Loop will consume one or more Segments.
        """
        self.logger.debug("parse(%s, source)", self.cls.__name__)
        arg_dict = {}
        for name, loop_type in class_fields(self.cls):
            self.logger.debug("parse %s: %r %s", name, loop_type, type(loop_type))
            arg_dict[name] = self.msg_attr_build(self.cls, name, loop_type, source)
        return self.cls(**arg_dict)

    def msg_attr_build(self, cls: type[Message], name: str, loop_type: type, source: Source) -> Loop | list[Any] | None:
        """
        Walks type structure to create Message of Loop instances.

        May consume source Segments.

        Return type can be a tricky recursive structure; we use list[Any]
        """
        self.logger.debug("msg_attr_build cls=%s, %s: %s", cls, name, loop_type)
        match loop_type:
            case GenericAlias() if get_origin(loop_type) is list:
                # list[Loop] structure
                repeating_type = get_args(loop_type)[0]
                values: list[Loop | list[Any] | None] = []
                next_value = self.msg_attr_build(cls, name, repeating_type, source)
                while next_value:
                    values.append(next_value)
                    next_value = self.msg_attr_build(cls, name, repeating_type, source)
                return values
            case _AnnotatedAlias():  # type: ignore[misc]
                base_type = get_args(loop_type)[0]
                return self.msg_attr_build(cls, name, base_type, source)
            case UnionType() | _UnionGenericAlias():  # type: ignore[misc]
                # Something | None OR list[Something] | None
                alternatives = list(filter(lambda t: t is not NoneType, get_args(loop_type)))
                union_type = alternatives[0]
                return self.msg_attr_build(cls, name, union_type, source)
            case type() if issubclass(loop_type, Loop):
                # Stand-alone Loop without a list[Loop] framing...
                value = self.loop_parse(loop_type, source)
                return value
        raise TypeError(f"unexpected {name}: {loop_type} {type(loop_type)}")  # pragma: no cover

    def loop_parse(self, cls: type[Loop], source: Source) -> Union[Loop, None]:
        """
        Parses the Exchange format text for this Loop subclass.
        """
        self.logger.debug("loop_parse(%s) %s", cls.__name__, cls)
        arg_dict = {}
        for name, field_type in class_fields(cls):
            self.logger.debug("loop_parse %s: %r", name, field_type)
            loop_value = self.loop_attr_build(cls, name, field_type, source)
            # Implicitly treat a loop as "Situational": ignore it if it contains no Segments.
            if loop_value:
                arg_dict[name] = loop_value
        if arg_dict:
            return cls(**arg_dict)
        else:
            self.logger.debug("Skipping empty loop %s", cls)
            return None

    def loop_attr_build(self, cls: type[Loop], name: str, field_type: type, source: Source) -> Loop | Segment | list[Any] | None:
        """
        Walks type structure to create Loop of Loops and Segments.

        May consume source Segments.

        Return type is a tricky recursive structure; we use list[Any]
        """
        self.logger.debug("loop_attr_build %s: %r", name, field_type)
        match field_type:
            case GenericAlias() if get_origin(field_type) is list:
                # list[Something]
                repeating_type = get_args(field_type)[0]
                self.logger.debug("  list[%s] %s", repeating_type.__name__, repeating_type)
                values: list[Loop | Segment | list[Any] | None] = []
                next_value = self.loop_attr_build(cls, name, repeating_type, source)
                while next_value:
                    values.append(next_value)
                    # next_value = self.segment_parse(repeating_type, source)
                    next_value = self.loop_attr_build(cls, name, repeating_type, source)
                return values
            case UnionType() | _UnionGenericAlias():  # type: ignore[misc]
                # Something | None OR list[Something] | None
                alternatives = list(filter(lambda t: t is not NoneType, get_args(field_type)))
                # OR. Try each alternative until one works.
                # For None supply a None instance.
                if len(alternatives) != 1:
                    raise NotImplementedError(f"not supported {field_type}; too many {alternatives}")  # pragma: no cover
                union_type = alternatives[0]
                return self.loop_attr_build(cls, name, union_type, source)
            case _AnnotatedAlias():  # type: ignore[misc]
                # Annotated[base, ...]
                base_type = get_args(field_type)[0]
                return self.loop_attr_build(cls, name, base_type, source)
            case type() if issubclass(field_type, Segment):
                seg_name = source.next_segment()
                if seg_name == "":
                    return None
                # TODO: segment with usage == "R"? Error
                # Else move on to next potential segment of Loop?
                return self.segment_parse(field_type, source)
            case type() if issubclass(field_type, Loop):  # pragma: no cover
                # raise RuntimeError("unexpected nested Loop: {field_type} inside {cls.__name__}")
                # An alternative is to try to parse it...
                return self.loop_parse(field_type, source)
        raise TypeError(f"unexpected {field_type} {type(field_type)}")  # pragma: no cover

    def segment_parse(self, cls: type[Segment], source: Source) -> Union[Segment, None]:
        """
        Parses the Exchange format text for this Segment subclass.

        Handles ISA segments specially to extract element and segment separators.
        If the ISA is uncompressed, then the separator characters for the :py:class`Source` can be deduced;
        this updates the state of the :py:class`Source`, and all subsequent segments can be compressed.
        If the ISA is compressed, then the separator characters must be provided before parsing can start.

        If the next segment in the :py:class:`Source` matches
        this segment's ``_segment_name`` attribute, then the segment
        will be consumed. Each element and composite will be validated.

        If the next segment does not match the ``_segment_name`` attribute,
        this segment's value will be ``None``.
        """
        fields = get_type_hints(cls, include_extras=True)
        self.logger.debug("segment_parse %s with %s", cls.__name__, fields)
        if source.next_segment().upper() != cls._segment_name.upper():
            return None

        # Special case: the ISA segment can update the ``Source`` parsing state.
        # Used in the case when ISA header not compressed.
        # If the ISA header is compressed, the separators must be provided "manually" to the :py:class:`Source` instance.
        if cls._segment_name.upper() == "ISA" and not source.element_sep:
            # Sum of field size + separator for each field.
            # + 3 for header "ISA"
            # + 1 for final segment separator
            args: list[tuple[X12Annotation]] = cast(list[tuple[X12Annotation]], [get_args(hint)[1:] for name, hint in class_fields(cls)])
            max_len_params: list[int] = [
                ann.params[0]
                for arg in args
                if arg
                    for ann in arg
                    if isinstance(ann, MaxLen)
            ]
            # print(f"Segment.parse() {max_len_params}")
            size = sum(p+1 for p in max_len_params) + 4
            self.logger.debug("ISA: source.element_sep=%r, source.segment_sep=%r", source.element_sep, source.segment_sep)
            self.logger.debug("ISA: size=%s", size)  # Should be 106 to include segment separator
            raw_text = source.peek(size)
            self.logger.debug("ISA: text=%r, separators text[-3:]=%r", raw_text, raw_text[-3:])
            # TODO: Rething this hack
            # ISA16 has the composite separator.
            # It's preceeded by an element separator and followed by the segment separator.
            source.element_sep = raw_text[-3]
            source.array_sep = raw_text[-2]
            source.segment_sep = raw_text[-1]

        # Note: Transition from parsing to building.
        # Extract all element values and build the segment.
        flat_values = cast(list[str | None | list[str | None]], source.elements())[1:]  # Drops the segment name from the values.
        self.logger.debug("segment_parse %s: fields %s values %r", cls.__name__, fields, flat_values)
        arg_dict: dict[str, Any] = {
            name: self.segment_attr_build(field_type, name, field_type, flat_values)
            for name, field_type in class_fields(cls)
        }
        obj = cls(**arg_dict)
        return obj

    def segment_attr_build(self, cls: type[Segment], name: str, field_type: type[Segment], source_values: list[str | None | list[str | None]]) -> Any | list[Any] | Composite | None:
        """
        Walks type structure to create Elements and Composites. The Element's are ``Any`` type.

        May pop source_values as part of building the instance.
        """
        self.logger.debug("segment_attr_build %s: %s from %r", name, field_type, source_values)
        match field_type:
            case UnionType() | _UnionGenericAlias():  # type: ignore[misc]
                alternatives = list(filter(lambda t: t is not NoneType, get_args(field_type)))
                if len(alternatives) != 1:
                    raise NotImplementedError(f"not supported {field_type}; too many {alternatives}")  # pragma: no cover
                if source_values:
                    return self.segment_attr_build(cls, name, alternatives[0], source_values)
                return self.segment_attr_build(cls, name, alternatives[0], [None])
            case _AnnotatedAlias():  # type: ignore[misc]
                # Consume a single value from the elements of this segment.
                # Either a ``Composite`` or list or some primitive type
                base = get_args(field_type)[0]
                value = self.segment_attr_build(cls, name, base, source_values)
                return value
            case GenericAlias() if get_origin(field_type) is list: # list[Something]
                # COMPOSITE-LIKE PROCESSING for elements that are list[Element]
                repeating_type = get_args(field_type)[0]
                self.logger.debug("  list[%s]", repeating_type.__name__)
                source_array = cast(list[str], source_values.pop(0))
                values = [self.segment_attr_build(cls, name, repeating_type, [item]) for item in source_array]
                return values
            case type() if issubclass(field_type, Composite):
                # Next value in this segment *should* be a list.
                # If it didn't have the component separator, we make single-element list.
                composite_value: list[str | None]
                if source_values:
                    composite_value = cast(list[str | None], source_values.pop(0))
                else:
                    composite_value = []
                # print(f"Segment.attr_build() {field_type} with {composite_value}")
                value = self.composite_build(field_type, composite_value)
                return value
            case type():
                # Atomic elements.
                if source_values:
                    raw_value = cast(str, source_values.pop(0))
                    return raw_value
                return None
        raise TypeError(f"unexpected type {field_type} {type(field_type)} in {cls}")  # pragma: no cover

    def composite_build(self, cls: type[Composite], source_value: str | None | list[str | None]) -> Composite:
        """
        Builds an instance of this Composite using a
        collection of source values extracted from parent Segment's Exchange format text.

        :param cls: the ``Composite`` subclass to build.
        :param source_value: the source values to use.
        :returns: an instance of this ``Composite`` subclass.
        """
        fields = get_type_hints(cls, include_extras=True)
        self.logger.debug("composite_build %s with %s", cls.__name__, fields)
        # Expand single value to a list
        if not isinstance(source_value, list):
            source_value = [source_value]

        # Pad with None instances?
        num_fields = sum(1 for name in class_fields(cls))
        if num_fields < len(source_value):
            # Too many values for the fields of this composite.
            raise ValueError(
                f"wrong number of values {cls.__name__} needs {num_fields} provided {len(source_value)}"
            )  # pragma: no cover
        elif num_fields > len(source_value):
            source_value.extend([None] * (num_fields - len(source_value)))
        else:
            # assert num_fields == len(source_value)
            pass
        # Build the object
        arg_dict = {
            name: self.composite_attr_build(cls, name, field_type, source_value.pop(0))
            for name, field_type in class_fields(cls)
        }
        return cls(**arg_dict)

    def composite_attr_build(self, cls: type[Composite], name: str, field_type: type, source_value: str | None) -> Any:
        """
        Populate Element's field value from a text value.
        """
        match field_type:
            case _AnnotatedAlias():  # type: ignore[misc]
                # Annotated[X, ...]
                return source_value
            case UnionType() | _UnionGenericAlias():  # type: ignore[misc]
                # Usually, X | None
                return source_value
            case type():
                return source_value
        raise TypeError(f"unexpected type {field_type}")  # pragma: no cover

def demo() -> None:
    """
    A minimal demonstration of some features of the base classes.

    ::

        >>> from x12.base import demo
        >>> demo()
        MSG(isa_loop=[ISA_LOOP(isa=ISA_LOOP_ISA(isa01='00', isa16=':'))])
        MSG(isa_loop=[ISA_LOOP(isa=ISA_LOOP_ISA(isa01='00', isa16=':'))])
    """

    ID: TypeAlias = str
    AN: TypeAlias = str

    class ISA_LOOP_ISA(Segment):
        _segment_name = "ISA"
        isa01: Annotated[ID, Title('I01'), MinLen(2), MaxLen(2)]
        isa16: Annotated[AN, Title('I16'), MinLen(1), MaxLen(1)]

    class ISA_LOOP(Loop):
        isa: ISA_LOOP_ISA

    class MSG(Message):
        """Demonstration Message"""
        isa_loop: list[ISA_LOOP]

    text = "ISA|00|:~"
    logger.info("Building from %r", text)
    source = Source(text, element_sep="|", segment_sep="~")
    parser = X12Parser(MSG)
    parsed_m = parser.parse(source)
    print(parsed_m)

    built_m = MSG(
        isa_loop=[
            ISA_LOOP(
                isa=ISA_LOOP_ISA(
                    isa01='00',
                    # etc.
                    isa16=':',
                )
            )
        ]
    )
    print(built_m)


if __name__ == "__main__":  # pragma: no cover
    logging.basicConfig(level=logging.DEBUG, stream=sys.stderr)
    demo()
    logging.shutdown()
